from django.db import models
from rest_framework.fields import FileField
from vaultier.business.fields import PermsField
from django.core.exceptions import ValidationError


class SecretPermsField(PermsField):

    def get_acls(self, obj, user):
        return obj.card.acl_set.filter(
            user=user
        )


class SecretTypeField(models.IntegerField):

    SECRET_TYPE_NOTE = 100
    SECRET_TYPE_PASSWORD = 200
    SECRET_TYPE_FILE = 300

    SECRET_TYPE_CHOICES = (
        (SECRET_TYPE_NOTE, 'NOTE'),
        (SECRET_TYPE_FILE, 'FILE'),
        (SECRET_TYPE_PASSWORD, 'PASSWORD')
    )

    def __init__(self, *args, **kwargs):
        kwargs['choices'] = self.SECRET_TYPE_CHOICES
        super(SecretTypeField, self).__init__(*args, **kwargs)


class BlobDataField(FileField):

    def to_native(self, value):
        return value.read()

    def from_native(self, data):
        max_size = 100 * 1000
        if not data:
            raise ValidationError('At least blob_data must be specified')

        if data and hasattr(data, 'size') and data.size > max_size:
            raise ValidationError('Maximum blob size is 100K encrypted')

        if data and not hasattr(data, 'size') and len(data) > max_size:
            raise ValidationError('Maximum blob size is 100K encrypted')

        return super(BlobDataField, self).from_native(data)



import datetime
from django.core.management.base import BaseCommand
from news.business.news_puller import puller
from django.core.cache import cache
from random import randint


class Command(BaseCommand):
    args = '<news count>'
    help = 'Creates new dummy messages for testing'

    def handle(self, *args, **options):

        try:
            count = args[0]
        except IndexError:
            count = 3

        data = []
        for _ in xrange(0, count):
            data.append({
                'id': randint(0, 999),
                'text': 'Description Description Description Description '
                        'Description',
                'title': 'Needs to do an upgrade to newer version. '
                         'Then you have to reboot your server.',
                'link': 'https://vaultier.org/blog',
                'published_at': datetime.datetime.now()
            })
        cache.set(puller.DATA_KEY, data)
        self.stdout.write('News dummy data successfully created')



from django.db import models
from django.db.models.query import QuerySet
from django.utils.timezone import now
from libs.changes.changes import post_change, SOFT_DELETE


class SoftDeleteManagerMixin(object):
    def get_queryset(self):
        """
        Returns a new QuerySet object.  Subclasses can override this method
        to easily customize the behavior of the Manager.
        """
        return QuerySet(self.model, using=self._db).filter(deleted_at=None)

    def include_deleted(self):
        return QuerySet(self.model, using=self._db)


class SoftDeleteMixin(models.Model):
    class Meta:
        abstract = True

    deleted_at = models.DateTimeField(null=True)

    @classmethod
    def bulk_delete(cls, queryset):
        """
        Soft delete a query select
        :param queryset: QuerySet
        :return:
        """
        queryset.values('pk').update(deleted_at=now())

    def softdelete(self, using=None):

        try:
            self.set_post_change_signal_enabled(False)
            self.deleted_at = now()
            self.save()
        finally:
            self.set_post_change_signal_enabled(True)

        if self._post_change_signal_disabled == 0:
            post_change.send(
                sender=self.__class__,
                instance=self,
                event_type=SOFT_DELETE,
                overwritten_values=self.overwritten_values()
            )



from .invitation import invitation_suite
from .workspace import workspace_suite
from .workspace_perms import workspace_perms_suite
from .workspace_softdelete import workspace_softdelete_suite
from .workspace_version import workspace_version_suite
from .workspacekey import workspacekey_suite
from django.utils.unittest.suite import TestSuite


def suite():
    s = TestSuite()
    s.addTest(workspace_suite())
    s.addTest(workspace_perms_suite())
    s.addTest(workspace_softdelete_suite())
    s.addTest(workspace_version_suite())
    s.addTest(workspacekey_suite())
    return s



import requests
import json
from django.conf import settings
from accounts.models import User, Member
from cards.models import Card
from secrets.models import Secret
from vaults.models import Vault
from workspaces.models import Workspace


class StatisticsManager(object):
    """
    Collects statistics about count of Vaultier objects and sends them
    to central endpoint.
    """
    STATISTICS_ENDPOINT = 'http://www.vaultier.org/api/ping'

    @staticmethod
    def send_statistics():

        enabled = settings.VAULTIER.get('allow_anonymous_usage_statistics')
        if not enabled:
            return

        workspace_count = Workspace.objects.all().count()
        vault_count = Vault.objects.all().count()
        card_count = Card.objects.all().count()
        secret_count = Secret.objects.all().count()
        user_count = User.objects.all().count()
        member_count = Member.objects.all().count()

        payload = {'workspaces': workspace_count,
                   'vaults': vault_count,
                   'cards': card_count,
                   'secrets': secret_count,
                   'users': user_count,
                   'members': member_count}
        requests.post(StatisticsManager.STATISTICS_ENDPOINT,
                      data=json.dumps(payload))



from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import ModelViewSet
from accounts.business.authentication import TokenAuthentication
from cards.business.permissions import CanManageCardPermission
from cards.models import Card
from cards.serializers import CardSerializer
from libs.version.context import VersionContextAwareApiViewMixin
from vaultier.business.mixins import AtomicTransactionMixin, \
    RetrieveBySlugMixin, SoftDeleteModelMixin


class CardViewSet(AtomicTransactionMixin,
                  RetrieveBySlugMixin,
                  SoftDeleteModelMixin,
                  VersionContextAwareApiViewMixin,
                  ModelViewSet):
    """
    API endpoint that allows cars to be viewed or edited.
    """
    model = Card
    serializer_class = CardSerializer
    authentication_classes = (TokenAuthentication, )
    permission_classes = (IsAuthenticated, CanManageCardPermission)
    filter_fields = ('vault',)

    def pre_save(self, object):
        if object.pk is None:
            self.check_object_permissions(self.request, object)
            object.created_by = self.request.user
        return super(CardViewSet, self).pre_save(object)

    def get_queryset(self):
        if self.action == 'list':
            queryset = Card.objects.all_for_user(self.request.user)
        else:
            queryset = Card.objects.all()
        return queryset



import os
from django.db import models
from django.db.models.deletion import PROTECT, CASCADE
from django.conf import settings
from libs.softdelete.softdelete import SoftDeleteMixin
from libs.tree.iterator import TreeIterableModelMixin
from secrets.business.fields import SecretTypeField
from secrets.business.managers import SecretManager
from libs.changes.changes import ChangesMixin
from secrets.business.tree import SecretTreeIterator


def get_blob_data_filename(instance, filename):
    path = settings.MEDIA_ROOT
    id = str(instance.id)
    subpath = '/'.join(list(id))
    fname = 'secret_'+str(instance.id)+'.encrypted.bin'
    return os.path.join(path, subpath, fname)


class Secret(ChangesMixin, SoftDeleteMixin, TreeIterableModelMixin,
             models.Model):

    tree_iterator_class = SecretTreeIterator

    objects = SecretManager()

    name = models.CharField(max_length=255, default='', blank=True, null=True)

    type = SecretTypeField()

    data = models.TextField(null=True, blank=True)
    blob_data = models.FileField(upload_to=get_blob_data_filename, null=True,
                                 blank=True)
    blob_meta = models.TextField(null=True, blank=True)
    card = models.ForeignKey('cards.Card', on_delete=CASCADE)
    created_by = models.ForeignKey('accounts.User', on_delete=PROTECT)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        db_table = u'vaultier_secret'



from django.core.urlresolvers import reverse

from vaultier.test.tools import VaultierAPIClient


def create_workspace_api_call(token, **kwargs):
    url = reverse('workspace-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.post(url, kwargs)
    return response


def delete_workspace_api_call(token, id):
    url = reverse('workspace-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.delete(url)
    return response


def retrieve_workspace_api_call(token, id):
    url = reverse('workspace-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def list_workspaces_api_call(token):
    url = reverse('workspace-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def update_workspace_api_call(token, id, **kwargs):
    url = reverse('workspace-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.put(url, kwargs)
    return response



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'Slug'
        db.create_table(u'vaultier_slug', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('slug', self.gf('django.db.models.fields.CharField')(unique=True, max_length=255, db_index=True)),
            ('content_type', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['contenttypes.ContentType'])),
            ('object_id', self.gf('django.db.models.fields.PositiveIntegerField')()),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
        ))
        db.send_create_signal(u'slugs', ['Slug'])


    def backwards(self, orm):
        # Deleting model 'Slug'
        db.delete_table(u'vaultier_slug')


    models = {
        u'contenttypes.contenttype': {
            'Meta': {'ordering': "('name',)", 'unique_together': "(('app_label', 'model'),)", 'object_name': 'ContentType', 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        u'slugs.slug': {
            'Meta': {'object_name': 'Slug', 'db_table': "u'vaultier_slug'"},
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['contenttypes.ContentType']"}),
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'object_id': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'slug': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '255', 'db_index': 'True'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['slugs']


from datetime import timedelta
from django.test.testcases import TransactionTestCase
from django.conf import settings
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_201_CREATED, HTTP_200_OK, \
    HTTP_403_FORBIDDEN, HTTP_400_BAD_REQUEST
from accounts.business.authentication import Backend
from accounts.models import Token
from libs.version.context import version_context_manager
from .api import auth_api_call, register_api_call
from vaultier.test.tools import FileAccessMixin, format_response
from django.utils import timezone


class SignaturesTest(TransactionTestCase, FileAccessMixin):

    def test_rsa(self):
        email = 'jan.misek@rclick.cz'
        privkey = self.read_file('vaultier.key')
        pubkey = self.read_file('vaultier.pub')
        signature = Backend.sign(privkey, email, 1)

        self.assertTrue(Backend.verify(pubkey, email, 1, signature))
        self.assertFalse(Backend.verify(pubkey, 'Unsigned text', 1, signature))


class ApiRegisterTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_010_register(self):
        # register user
        email = 'jan.misek@rclick.cz'
        response = register_api_call(email=email, nickname='Misan')
        self.id = response.data.get('id')
        self.assertEqual(response.status_code, HTTP_201_CREATED,
                         msg=format_response(response))

        # register user again should be forbidden
        response = register_api_call(email=email, nickname='Misan')
        self.assertTrue(response.data.get('email') is not None)
        self.assertEqual(response.status_code, HTTP_400_BAD_REQUEST,
                         msg=format_response(response))

    def test_020_auth(self):
        # register user
        email = 'jan.misek@rclick.cz'
        response = register_api_call(email=email, nickname='Misan')
        self.id = response.data.get('id')
        self.assertEqual(response.status_code, HTTP_201_CREATED,
                         msg=format_response(response))

        # try to login, check proper signature
        response = auth_api_call(email=email)
        self.assertEqual(response.status_code, HTTP_200_OK,
                         msg=format_response(response))

        # try to login, check wrong signature
        response = auth_api_call(email=email, signature='WrongSignature')
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         msg=format_response(response))

        # try to login, check wrong expired timestamp
        expired = timezone.now() - timedelta(days=15)
        response = auth_api_call(email=email, date=expired,
                                 signature='WrongSignature')
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         msg=format_response(response))

    def test_030_registration_should_be_case_insensitive(self):
        # register user
        email = 'TeSt.CaSe.InSeSiTiVe@rclick.cz'
        response = register_api_call(email=email, nickname='case_insensitive')
        self.id = response.data.get('id')
        self.assertEqual(response.status_code, HTTP_201_CREATED,
                         msg=format_response(response))

        # login with the email to lower case
        response = auth_api_call(email=email.lower())
        self.assertEqual(response.status_code, HTTP_200_OK,
                         msg=format_response(response))
        self.assertEqual(response.data.get('user'), self.id,
                         'User could be logged with the LOWER case '
                         'version of the email'
                         )

        response = auth_api_call(email=email.upper())
        self.assertEqual(response.status_code, HTTP_200_OK,
                         msg=format_response(response)
                         )
        self.assertEqual(response.data.get('user'), self.id,
                         'User could be logged with the UPPER case '
                         'version of the email'
                         )

        # register user
        email = 'TEST_UPPERCASE_REGISTRATION@rclick.cz'
        response = register_api_call(email=email,
                                     nickname='UPPERCASE_REGISTRATION')
        self.id = response.data.get('id')
        self.assertEqual(response.status_code,
                         HTTP_201_CREATED,
                         msg=format_response(response)
                         )

        # login with the email to lower case
        response = auth_api_call(email=email.lower())
        self.assertEqual(response.status_code,
                         HTTP_200_OK,
                         msg=format_response(response)
                         )

    def test_040_email_cannot_be_registered_more_than_once(self):
        # register user
        email = 'test.040@rclick.cz'
        response = register_api_call(email=email, nickname='test040')
        self.id = response.data.get('id')
        self.assertEqual(response.status_code,
                         HTTP_201_CREATED,
                         msg=format_response(response)
                         )

        email = 'TEST.040@rclick.cz'
        response = register_api_call(email=email, nickname='test040')
        self.id = response.data.get('id')
        self.assertEqual(response.status_code,
                         HTTP_400_BAD_REQUEST,
                         msg=format_response(response)
                         )
        self.assertEqual(response.data.get('email'),
                         [u'User with this Email already exists.'],
                         'The email was already registered'
                         )
        # register user
        email = 'TEST_040@rclick.cz'
        response = register_api_call(email=email, nickname='test040')
        self.id = response.data.get('id')
        self.assertEqual(response.status_code,
                         HTTP_201_CREATED,
                         msg=format_response(response)
                         )

        email = 'test_040@rclick.cz'
        response = register_api_call(email=email, nickname='test040')
        self.id = response.data.get('id')
        self.assertEqual(response.status_code, HTTP_400_BAD_REQUEST,
                         msg=format_response(response))
        self.assertEqual(response.data.get('email'),
                         [u'User with this Email already exists.'],
                         'The email was already registered')


class TokenExpirationTest(TransactionTestCase):

    def test_010_token_renewed(self):
        # register user
        email = 'jan.misek@rclick.cz'
        register_api_call(email=email, nickname='Misan')
        response = auth_api_call(email=email)
        token = response.data.get('token')

        token_model = Token.objects.get(token=token)
        """:type : Token """
        td = timedelta(days=-1)
        last_used_at = timezone.now() - td
        token_model.last_used_at = last_used_at
        token_model.save()

        # current date should be set, limit 60 seconds in case of some slowdown
        self.assertTrue(
            (timezone.now() - token_model.last_used_at).total_seconds() < 60)

    def test_020_token_not_renewed_immediately(self):
        # register user
        email = 'jan.misek@rclick.cz'
        register_api_call(email=email, nickname='Misan')
        response = auth_api_call(email=email)
        token = response.data.get('token')

        token_model = Token.objects.get(token=token)
        """:type : Token """

        last_used_at = token_model.last_used_at
        token_model.save()

        # last_used_at not changed
        self.assertEqual(last_used_at, token_model.last_used_at)

    def test_030_old_token_cleaned_up(self):
        # register user
        email = 'jan.misek@rclick.cz'
        register_api_call(email=email, nickname='Misan')
        response = auth_api_call(email=email)
        token = response.data.get('token')

        token_model = Token.objects.get(token=token)
        """:type : Token """
        token_lifetime = settings.VAULTIER.get('authentication_token_lifetime')
        td = timedelta(hours=2 * token_lifetime)
        last_used_at = timezone.now() - td
        token_model.last_used_at = last_used_at
        token_model.save()

        Token.objects.clean_old_tokens()
        self.assertEqual(Token.objects.all().count(), 0)

    def test_040_recent_token_not_cleaned_up(self):
        # register user
        email = 'jan.misek@rclick.cz'
        register_api_call(email=email, nickname='Misan')
        response = auth_api_call(email=email)
        token = response.data.get('token')

        token_model = Token.objects.get(token=token)
        """:type : Token """
        token_lifetime = settings.VAULTIER.get('authentication_token_lifetime')
        td = timedelta(hours=token_lifetime - 1)
        last_used_at = timezone.now() - td

        token_model.last_used_at = last_used_at
        token_model.save()

        Token.objects.clean_old_tokens()
        # token not deleted
        self.assertEqual(Token.objects.all().count(), 1)


def test_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(SignaturesTest))
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ApiRegisterTest))
    suite.addTest(
        unittest.TestLoader().loadTestsFromTestCase(TokenExpirationTest))
    return suite



from rest_framework import serializers
from cards.serializers import CardSerializer
from vaults.serializers import VaultSerializer, RelatedVaultSerializer
from workspaces.serializers import RelatedWorkspaceSerializer


class SearchSerializer(serializers.Serializer):
    query = serializers.CharField(required=True, max_length=100, min_length=1)
    type = serializers.CharField(required=False, max_length=100, min_length=1)


class SearchVaultSerializer(VaultSerializer):
    workspace = RelatedWorkspaceSerializer()


class SearchCardSerializer(CardSerializer):
    workspace = serializers.SerializerMethodField('get_workspace')
    vault = RelatedVaultSerializer()

    def get_workspace(self, obj):
        return RelatedWorkspaceSerializer(instance=obj.vault.workspace).data

    class Meta(CardSerializer.Meta):
        fields = CardSerializer.Meta.fields + ('workspace',)


class SearchResultSerializer(serializers.Serializer):
    def to_native(self, obj):
        result = {
            'vaults': [],
            'cards': []
        }

        vaults = obj.get('vaults')
        if vaults and len(vaults):
            for vault in vaults:
                result.get('vaults').append(
                    SearchVaultSerializer(instance=vault,
                                          context=self.context).data)

        cards = obj.get('cards')
        if cards and len(cards):
            for card in cards:
                result.get('cards').append(
                    SearchCardSerializer(instance=card,
                                         context=self.context).data)

        return result



from django.core.urlresolvers import reverse

from vaultier.test.tools import VaultierAPIClient


def invite_member_api_call(token, email=None, workspace=None, send=True,
                           resend=True):
    url = reverse('member-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.post(url, {
        'email': email,
        'workspace': workspace,
        'send': send,
        'resend': resend
    })
    return response


def list_members_api_call(token, workspace):
    url = reverse('member-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url, {'workspace': workspace})
    return response


def delete_member_api_call(token, member):
    url = reverse('member-detail', args=(member,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.delete(url)
    return response



import collections
import types
from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from libs.changes.changes import post_change, INSERT, UPDATE, DELETE
from libs.models import Garage


class ChangesMixinTest(TransactionTestCase):
    def assertDictEqual(self, d1, d2, msg=None):  # assertEqual uses for dicts
        for k, v1 in d1.iteritems():
            self.assertIn(k, d2, msg)
            v2 = d2[k]
            if (isinstance(v1, collections.Iterable) and
                    not isinstance(v1, basestring)):
                self.assertItemsEqual(v1, v2, msg)
            else:
                self.assertEqual(v1, v2, msg)
        return True

    def test_dirtiness(self):
        g = Garage(car1='skoda')

        # no dirty fields there
        v = g.dirty_values()
        self.assertEqual(0, len(v.keys()))

        #one dirty field should be there
        g.car2 = 'lada'
        v = g.dirty_values()
        self.assertDictEqual({'car2': ''}, v)

        #no dirty fields should be there
        g.save()
        v = g.dirty_values()
        self.assertDictEqual({}, v)

        #no dirty field should be there because we did not do any change
        g.car2 = 'lada'
        v = g.dirty_values()
        self.assertDictEqual({}, v)

        #one dirty field should be there because we did not do any change
        g.car2 = 'porche'
        v = g.dirty_values()
        self.assertDictEqual({'car2': 'lada'}, v)

    def test_previous(self):
        g = Garage(car1='skoda')
        g.save()

        # assert previous values
        v = g.previous_values()
        self.assertDictEqual({'car1': 'skoda', 'car2': '', 'id': None}, v)

        # assert overwritten values
        g.car2 = 'lada'
        g.save()
        v = g.overwritten_values()
        self.assertDictEqual({'car2': ''}, v)

    def test_signals(self):

        def on_change(sender=None, instance=None, overwritten_values=None,
                      event_type=None, **kwargs):
            self.signal_overwritten_values = overwritten_values
            self.signal_event_type = event_type
        post_change.connect(on_change, sender=Garage)

        g = Garage(car1='skoda')
        g.save()

        self.assertDictEqual({'id': None}, self.signal_overwritten_values)
        self.assertEquals(self.signal_event_type, INSERT)

        g.car1 = 'fiat'
        g.save()

        self.assertDictEqual({'car1': 'skoda'}, self.signal_overwritten_values)
        self.assertEquals(self.signal_event_type, UPDATE)

        g.delete()

        self.assertDictEqual({}, self.signal_overwritten_values)
        self.assertEquals(self.signal_event_type, DELETE)

    def test_hooks(self):

        g = Garage(car1='skoda')
        g.pre_save_hook = None
        g.post_change_hook = None

        def pre_save(self, **kwargs):
            self.pre_save_hook = 'ok'

        def post_change(self, **kwargs):
            self.post_change_hook = 'ok'

        g.on_pre_save = types.MethodType(pre_save, g)
        g.on_post_change = types.MethodType(post_change, g)

        g.save()
        self.assertEqual(g.post_change_hook, 'ok')
        self.assertEqual(g.pre_save_hook, 'ok')


def changes_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        ChangesMixinTest))
    return suite



from libs.version.context import version_context_manager


class BasicCondition(object):
    def will_do_version(
            self, signal=None, sender=None, instance=None, event_type=None,
            overwritten_values=None, **kwargs):
        return version_context_manager.get_enabled()


class RequiredFieldEventCondition(BasicCondition):
    required_sender = None
    required_event = None
    required_fields = None

    def __init__(self, required_fields=[], required_event=None,
                 required_sender=None, *args, **kwargs):
        self.required_fields = required_fields
        self.required_event = required_event
        self.required_sender = required_sender

    def will_do_version(
            self, signal=None, sender=None, instance=None, event_type=None,
            overwritten_values=None, **kwargs):
        basic = super(RequiredFieldEventCondition, self).will_do_version(
            signal=signal,
            sender=sender,
            instance=instance,
            event_type=event_type,
            overwritten_values=overwritten_values,
            **kwargs
        )

        if basic \
            and event_type == self.required_event \
                and sender == self.required_sender:

            saved_keys = overwritten_values.keys()
            do_save = False
            intersection = {}
            if self.required_fields:
                if set(saved_keys).intersection(self.required_fields):
                    do_save = True
                    # required fields specified, intersection only
                    # required fields
                    for saved_key in saved_keys:
                        if saved_key in self.required_fields:
                            intersection[saved_key] = \
                                overwritten_values[saved_key]
            else:
                # no required fields specifed, intersection is whole save state
                do_save = True
                intersection = overwritten_values

            if do_save:
                return intersection

        return None



from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import ModelViewSet
from acls.business.permissions import CanManageRolePermission
from acls.models import Role
from acls.serializers import RoleSerializer, RoleUpdateSerializer, \
    MemberRolesSerializer
from vaultier.business.mixins import AtomicTransactionMixin


class RoleViewSet(AtomicTransactionMixin, ModelViewSet):
    model = Role
    permission_classes = (IsAuthenticated, CanManageRolePermission)
    filter_fields = ('to_workspace', 'to_vault', 'to_card', 'level',)
    serializer_class = RoleSerializer

    def get_serializer_class(self):
        if self.action == 'update' or self.action == 'partial_update':
            return RoleUpdateSerializer
        elif self.action == 'list' and \
                self.request.QUERY_PARAMS.get('to_member', None):
            return MemberRolesSerializer
        else:
            return super(RoleViewSet, self).get_serializer_class()

    def pre_save(self, object):
        self.check_object_permissions(self.request, object)
        if not object.pk:
            object.created_by = self.request.user
        return super(RoleViewSet, self).pre_save(object)

    def get_queryset(self):
        member_id = self.request.QUERY_PARAMS.get('to_member')
        if member_id:
            queryset = Role.objects.all_for_member(member_id)
        else:
            queryset = Role.objects.all_for_user(self.request.user)
        return queryset



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_403_FORBIDDEN, HTTP_200_OK, \
    HTTP_204_NO_CONTENT
from accounts.models import Member
from accounts.tests.api import register_api_call, auth_api_call, \
    invite_member_api_call
from acls.business.fields import RoleLevelField
from acls.tests.api import create_role_api_call
from cards.tests.api import create_card_api_call, retrieve_card_api_call
from libs.version.context import version_context_manager
from secrets.business.fields import SecretTypeField
from secrets.tests.api import create_secret_api_call, delete_secret_api_call, \
    retrieve_secret_api_call, update_secret_api_call
from vaultier.test.tools import format_response
from vaults.tests.api import create_vault_api_call, retrieve_vault_api_call
from workspaces.tests.api import create_workspace_api_call, \
    accept_invitation_api_call, retrieve_workspace_api_call, \
    list_workspaces_api_call, delete_workspace_api_call


class ApiSecretPermsTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_000_secret_anonymous_access(self):
        response = create_secret_api_call(
            None, type=SecretTypeField.SECRET_TYPE_NOTE, card=None)
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

    def test_001_create_delete_secret_as_anonymous(self):

    # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(
            user1token, name='Workspace').data

        #create vault
        vault = create_vault_api_call(user1token, name="vault_in_workspace",
                                      workspace=workspace.get('id')).data

        #create card
        card = create_card_api_call(user1token, name="card_in_vault",
                                    vault=vault.get('id')).data

        #create secret
        secret = create_secret_api_call(
            user1token,
            type=SecretTypeField.SECRET_TYPE_NOTE,
            card=card.get('id')
        ).data

        #delete secret as anonymous
        response = delete_secret_api_call(None, secret.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

    @unittest.skip("should be fixed asap")
    def test_010_create_secret_in_card_without_acl_to_vault(self):
        # create user1
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(
            user1token, name='workspace').data

        # create vault for user1
        vault1 = create_vault_api_call(user1token, name='vault_in_workspace',
                                       workspace=workspace1.get('id')).data

        #create card
        card1 = create_card_api_call(user1token, name="card_in_vault",
                                     vault=vault1.get('id')).data

        # user2 tries to create secret in vault of user 2
        # which he has no access to
        response = create_card_api_call(
            user2token,
            type=SecretTypeField.SECRET_TYPE_NOTE,
            card=card1.get('id'))

        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

    @unittest.skip("should be fixed asap")
    def test_020_create_vault_and_and_check_permissions(self):
        # create user1
        email = 'jan@rclick.cz'
        user1 = register_api_call(email=email, nickname='Jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        user2 = register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(
            user1token, name='Workspace').data

        # create vault
        vault1 = create_vault_api_call(user1token, name="vault_in_workspace",
                                       workspace=workspace1.get('id')).data

        # create card
        card1 = create_card_api_call(user1token, name='card_in_vault',
                                     vault=vault1.get('id')).data

        secret1 = create_secret_api_call(
            user2token,
            type=SecretTypeField.SECRET_TYPE_NOTE,
            card=card1.get('id')
        ).data

        #user1 invites user and grant to user role READ to card1,
        # this should also grant role to secret1
        user2member = invite_member_api_call(
            user1token, email=user2.get('email'),
            workspace=workspace1.get('id')).data
        user2hash = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        user2accepted = accept_invitation_api_call(
            user2token, pk=user2member.get('id'), hash=user2hash)
        user2role = create_role_api_call(
            user1token,
            member=user2member.get('id'),
            to_card=card1.get('id'),
            level=RoleLevelField.LEVEL_READ
        )

        #user2 tries to read card, should be allowed
        response = retrieve_secret_api_call(user2token, secret1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #user2 tries to read card, should be allowed
        response = retrieve_card_api_call(user2token, card1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #user2 tries to read vault, should be allowed
        response = retrieve_vault_api_call(user2token, vault1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #user2 tries to read workspace, should be allowed
        response = retrieve_workspace_api_call(
            user2token, workspace1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        #user2 tries to delete secret, should be forbidden
        response = delete_secret_api_call(user2token, secret1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        #user2 tries to list workspace of vault1 of card1 of secret1,
        # should be allowed
        response = list_workspaces_api_call(user2token)
        self.assertEqual(
            len(response.data),
            1,
            format_response(response)
        )

        #user2 tries to delete workspace of vault1, should be forbidden
        response = delete_workspace_api_call(
            user2token, vault1.get('workspace'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        #user1 grants write to workspace, user2 should be able to delete secret
        response = create_role_api_call(
            user1token,
            member=user2member.get('id'),
            to_workspace=workspace1.get('id'),
            level=RoleLevelField.LEVEL_WRITE
        )

        #user2 tries to delete card, should be allowed
        response = delete_secret_api_call(user2token, secret1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_204_NO_CONTENT,
            format_response(response)
        )

    @unittest.skip("should be fixed asap")
    def test_030_retrieve_forbidden(self):
        # create user1
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        workspace1 = create_workspace_api_call(
            user1token, name='workspace').data

        # create vault for user1
        vault1 = create_vault_api_call(user1token, name='vault_in_workspace',
                                       workspace=workspace1.get('id')).data

        # create card1
        card1 = create_card_api_call(user1token, name="card_in_vault",
                                     vault=vault1.get('id')).data

        # create secret1
        secret1 = create_secret_api_call(
            user2token,
            type=SecretTypeField.SECRET_TYPE_NOTE,
            card=card1.get('id')
        ).data

        # user 2 tries to retrieve card1 should be forbidden
        response = retrieve_secret_api_call(user2token, secret1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

    @unittest.skip("should be fixed asap")
    def test_040_move_secret(self):
        # create user1
        email = 'jan@rclick.cz'
        user1 = register_api_call(email=email, nickname='Jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create user2
        email = 'marcel@rclick.cz'
        user2 = register_api_call(email=email, nickname='Marcel').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace1 = create_workspace_api_call(
            user1token, name='workspace').data

        # create vault for user1
        vault1 = create_vault_api_call(user1token, name='vault1',
                                       workspace=workspace1.get('id')).data

        # create card1
        card1 = create_card_api_call(
            user1token, name="card1", vault=vault1.get('id')).data

        # create card2
        card2 = create_card_api_call(
            user1token, name="card2", vault=vault1.get('id')).data

        #user1 invites user and grant to user role READ to card1,
        # this should also grant role to secret1
        user2member = invite_member_api_call(
            user1token, email=user2.get('email'),
            workspace=workspace1.get('id')).data
        user2hash = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        user2accepted = accept_invitation_api_call(
            user2token, pk=user2member.get('id'), hash=user2hash)
        user2role = create_role_api_call(
            user1token,
            member=user2member.get('id'),
            to_card=card1.get('id'),
            level=RoleLevelField.LEVEL_READ
        )

        # create secret1
        secret1 = create_secret_api_call(
            user1token,
            type=SecretTypeField.SECRET_TYPE_NOTE,
            card=card1.get('id')
        ).data

        # user 2 tries to retrieve card1 should be forbidden
        response = retrieve_secret_api_call(user2token, secret1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        # move secret to card2
        secret1 = update_secret_api_call(user1token, card=card2.get('id')).data

        # user 2 tries to retrieve card1 should be allowed
        response = retrieve_secret_api_call(user2token, secret1.get('id'))
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )


def secret_perms_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        ApiSecretPermsTest))
    return suite



from django.core.exceptions import ObjectDoesNotExist, ValidationError
from django.db import models
from django.utils.encoding import smart_text
from django.utils.translation import ugettext_lazy as _
from rest_framework.fields import Field
from rest_framework.relations import RelatedField
from acls.business.fields import AclLevelField


class ObjectReferenceTypeField(models.IntegerField):
    TYPE_WORKSPACE = 100
    TYPE_VAULT = 200
    TYPE_CARD = 300

    ACL_CHOICES = (
        (TYPE_WORKSPACE, 'WORKSPACE'),
        (TYPE_VAULT, 'VAULT'),
        (TYPE_CARD, 'CARD')
    )

    def __init__(self, *args, **kwargs):
        kwargs['choices'] = self.ACL_CHOICES
        super(ObjectReferenceTypeField, self).__init__(*args, **kwargs)


class PermsField(Field):
    read_only = True

    def get_acls(self, obj, user):
        acls = obj.acl_set.filter(
            user=user
        )
        return acls

    def field_to_native(self, obj, field_name):
        perms = {}

        user = self.context['request'].user
        acls = self.get_acls(obj, user)

        for acl in acls:
            if acl.level == AclLevelField.LEVEL_CREATE:
                perms['create'] = True
            if acl.level == AclLevelField.LEVEL_READ:
                perms['read'] = True
            if acl.level == AclLevelField.LEVEL_WRITE:
                perms['update'] = True
                perms['delete'] = True
                perms['create'] = True
                perms['invite'] = True
        return perms


class RelatedNestedField(RelatedField):
    serializer = None
    id_field = 'id'
    read_only = False

    default_error_messages = {
        'does_not_exist': _("Object with %s=%s does not exist."),
        'invalid': _('Invalid value.'),
    }

    def __init__(self, *args, **kwargs):
        self.id_field = kwargs.pop('id_field', self.id_field)
        self.serializer = kwargs.pop('serializer', None)
        assert self.serializer, 'serializer is required'
        super(RelatedNestedField, self).__init__(*args, **kwargs)

    def to_native(self, obj):
        return self.serializer().to_native(obj)

    def from_native(self, data):
        if self.queryset is None:
            msg = 'Writable related fields must include a `queryset` argument'
            raise Exception(msg)

        try:
            return self.queryset.get(**{self.id_field: data})
        except ObjectDoesNotExist:
            raise ValidationError(self.error_messages['does_not_exist'] %
                                  (self.id_field, smart_text(data)))
        except (TypeError, ValueError):
            msg = self.error_messages['invalid']
            raise ValidationError(msg)



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from libs.version.context import version_context_manager
from libs.version.manipulator import ACTION_CREATED, ACTION_UPDATED, \
    ACTION_SOFTDELETED
from versions.models import Version
from vaultier.test.tools.auth.api import auth_api_call, register_api_call
from vaultier.test.tools import AssertionsMixin
from vaultier.test.tools.vault.api import create_vault_api_call, \
    delete_vault_api_call, update_vault_api_call
from vaultier.test.tools.workspace.api import create_workspace_api_call


class VaultVersionTest(AssertionsMixin,  TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(True)

    def create_vault(self):
        # create user
        email = 'jan@rclick.cz'
        user1 = register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(
            user1token, name='workspace').data

        # create vault
        vault = create_vault_api_call(
            user1token, workspace=workspace.get('id'), name='vault').data

        return user1, user1token, workspace, vault

    @unittest.skip("should be fixed asap")
    def test_vault_010_create(self):
        user1, user1token, workspace, vault = list(self.create_vault())

        #check version
        versions = Version.objects.filter(
            versioned_type__model='vault',
            versioned_id=vault.get('id')
        )

        # one version should be there
        self.assertEquals(versions.count(), 1)

        # compare version
        self.assert_model(versions[0], {
            'versioned_id': vault.get('id'),
            'versioned_related_id': workspace.get('id'),
            'action_id': ACTION_CREATED
        })

        # assert user has been stored with version
        self.assertEqual(versions[0].created_by.id, user1.get('id'))

        # compare revision data
        self.assert_dict(versions[0].revert_data, {})

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'vault')
        self.assertEqual(versions[0].versioned_related_id, vault.get('id'))

    @unittest.skip("should be fixed asap")
    def test_vault_020_update(self):
        user1, user1token, workspace, vault = list(self.create_vault())

        vault = update_vault_api_call(
            user1token, vault.get('id'), name='renamed_vault',
            description="added_vault_description").data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='vault',
            versioned_id=vault.get('id'),
            action_id=ACTION_UPDATED
        )

        # one  versions should be there
        self.assertEquals(versions.count(), 1)

        # compare revision data
        self.assert_dict(versions[0].revert_data, {
            'name': 'vault',
            'description': None
        })

        vault = update_vault_api_call(
            user1token, vault.get('id'), name='renamed_vault_again',
            description="changed_vault_description").data

        #check version
        versions = Version.objects.filter(
            versioned_type__model='vault',
            versioned_id=vault.get('id'),
            action_id=ACTION_UPDATED
        )

        # two versions should be there
        self.assertEquals(versions.count(), 2)

        # compare revision data
        self.assert_dict(versions[1].revert_data, {
            'name': 'renamed_vault',
            'description': 'added_vault_description'
        })

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'vault')
        self.assertEqual(versions[0].versioned_related_id, vault.get('id'))

    @unittest.skip("should be fixed asap")
    def test_vault_030_delete(self):
        user1, user1token, workspace, vault = list(self.create_vault())

        #check version
        versions = Version.objects.filter(
            versioned_type__model='vault',
            versioned_id=vault.get('id'),
            action_id=ACTION_CREATED
        )

        # one  versions should be there
        self.assertEquals(versions.count(), 1)

        delete_vault_api_call(user1token, vault.get('id'))

        #check version
        versions = Version.objects.filter(
            versioned_type__model='vault',
            versioned_id=vault.get('id'),
            action_id=ACTION_SOFTDELETED
        )

        # two versions should be there, created and deleted
        self.assertEquals(versions.count(), 1)

        # compare revision data
        self.assert_dict(versions[0].revert_data, {'deleted_at': None})

        # assert related id
        self.assertEqual(versions[0].versioned_related_type.model, 'workspace')
        self.assertEqual(versions[0].versioned_related_id, workspace.get('id'))


def vault_version_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        VaultVersionTest))
    return suite



from rest_framework.response import Response
from rest_framework.views import APIView
from news.business.news_puller import puller
from news.serializers import NewsSerializer


class NewsApiView(APIView):
    """
    News api endpoint. Provides only list (GET) method
    """

    def get(self, request, *args, **kwargs):
        """
        Fetch data from NewsPuller service

        :param request:
        :param args:
        :param kwargs:
        :return:
        """
        data = puller.fetch()
        serializer = NewsSerializer(data, many=True)
        return Response(serializer.data)



from django.contrib.contenttypes.generic import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.db import models
from django.db.models.fields import PositiveIntegerField
from libs.changes.changes import post_change, INSERT, UPDATE, DELETE
from libs.slugify.slugify import unique_slugify


class SlugManagerMixin(object):
    def on_model(self, signal=None, sender=None, instance=None,
                 event_type=None, overwritten_values=None, **kwargs):
        try:
            instance.set_post_change_signal_enabled(False)

            # create slug with new instance
            if event_type == INSERT:
                self.create_slug_for_model(instance)

            # create slug when name of instance changed. Slugs are not
            # deleted (permalink pattern)
            if event_type == UPDATE and 'name' in overwritten_values:
                self.create_slug_for_model(instance)
            # handle only delete not softdelete
            if event_type == DELETE:
                self.delete_slugs(instance)
        finally:
            instance.set_post_change_signal_enabled(True)

    def create_slug_for_model(self, model):
        # prepare query set to generate slug
        model_type = ContentType.objects.get_for_model(model)
        queryset = self.get_queryset().exclude(
            object_id=model.id,
            content_type=model_type
        )

        # generate slugname
        slug_text = unique_slugify(model.name, queryset=queryset,
                                   default_slug=model.__class__.__name__)

        # if not slug exists, create
        if not self.get_queryset().filter(
                object_id=model.id,
                content_type=model_type,
                slug=slug_text
        ).count():
            s = self.create(content_object=model, slug=slug_text)
            s.save()

        # update slug on model
        model.slug = slug_text
        model.save()

    def delete_slugs(self, instance):
        """
        Deletes slugs related to instance
        @type instance: models.Model
        """
        c = ContentType.objects.get_for_model(instance)
        self.filter(object_id=instance.id, content_type=c).delete()


class SlugMixin(models.Model):
    slug = models.CharField(max_length=255, unique=True, db_index=True)

    content_type = models.ForeignKey(ContentType)
    object_id = PositiveIntegerField()
    content_object = GenericForeignKey('content_type', 'object_id')
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    class Meta:
        abstract = True


def register_signals(slug_model, slugable_model):
    post_change.connect(slug_model.objects.on_model, sender=slugable_model)



from django.core.urlresolvers import reverse

from vaultier.test.tools import VaultierAPIClient


def set_workspace_key_api_call(token, member, key):
    url = reverse('workspace_key-detail', args=(member,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.put(url, {'workspace_key': key})
    return response


def get_workspace_key_api_call(token, member):
    url = reverse('workspace_key-detail', args=(member,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response


def list_workspace_key_api_call(token):
    url = reverse('workspace_key-list')
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response



from unittest import TestSuite
from django.test.testcases import TransactionTestCase
from django.utils import unittest
from accounts.business.fields import MemberStatusField
from accounts.models import Member, User
from acls.business.fields import RoleLevelField, AclLevelField
from acls.models import Acl, Role
from cards.models import Card
from libs.version.context import version_context_manager
from vaults.models import Vault
from workspaces.models import Workspace


class AclTest(TransactionTestCase):

    def assertAcl(self, level=None, count=None, role=None, workspace=None,
                  vault=None, card=None):
        o = None
        if workspace:
            o = workspace
            if type(o) != Workspace:
                o = Workspace.objects.get(id=workspace.pk)
        if vault:
            o = vault
            if type(o) != Vault:
                o = Vault.objects.get(id=vault.pk)
        if card:
            o = card
            if type(o) != Card:
                o = Card.objects.get(id=card.pk)

        if not o:
            raise RuntimeError('Unknown test type')

        aclset = o.acl_set.all()

        if role:
            aclset = aclset.filter(role=role)

        if level:
            aclset = aclset.filter(level=level)

        if count is None:
            self.assertGreaterEqual(aclset.count(), 1)
        else:
            self.assertEquals(aclset.count(), count)

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_010_create_role(self):
        u = User()
        u.email = 'misan'
        u.save()

        w = Workspace()
        w._user = u
        w.name = 'workspace'
        w.created_by = u
        w.save()

        v = Vault()
        v._user = u
        v.name = 'vault'
        v.workspace = w
        v.created_by = u
        v.save()

        c = Card()
        c.name = 'vault'
        c.vault = v
        c.created_by = u
        c.save()

        m = Member()
        m.user = u
        m.status = MemberStatusField.STATUS_MEMBER
        m.workspace = w
        m.created_by = u
        m.save()

        # already materialized roles should be deleted to test only
        # create role
        Acl.objects.all().delete()
        Role.objects.all().delete()

        role = Role()
        role.level = RoleLevelField.LEVEL_WRITE
        role.created_by = u
        role.set_object(v)
        role.member = m
        role.save()

        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, workspace=w)

        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=1, vault=v)
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, vault=v)

        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=1, card=c)
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, card=c)

    def test_020_invited_member(self):
        u = User()
        u.email = 'misan'
        u.save()

        w = Workspace()
        w._user = u
        w.name = 'workspace'
        w.created_by = u
        w.save()

        # already materialized roles should be deleted
        Acl.objects.all().delete()
        Role.objects.all().delete()

        m = Member()
        m.workspace = w
        m.created_by = u
        m.save()

        role = Role()
        role.level = RoleLevelField.LEVEL_WRITE
        role.created_by = u
        role.set_object(w)
        role.member = m
        role.save()

        # 0 acls should be in db, because no materialization should happen
        # on invited member
        self.assertEquals(Acl.objects.all().count(), 0)

    def test_030_invited_member_become_regular(self):
        u = User()
        u.email = 'misan'
        u.save()

        w = Workspace()
        w._user = u
        w.name = 'workspace'
        w.created_by = u
        w.save()

        # already materialized roles should be deleted
        Acl.objects.all().delete()
        Role.objects.all().delete()

        m = Member()
        m.workspace = w
        m.created_by = u
        m.save()

        role = Role()
        role.level = RoleLevelField.LEVEL_WRITE
        role.created_by = u
        role.set_object(w)
        role.member = m
        role.save()

        m.user = u
        m.save()

        # 1 acls should be in db, because materialization should happen on
        # regular member
        self.assertAcl(level=AclLevelField.LEVEL_WRITE, role=role, count=1,
                       workspace=w)
        self.assertAcl(level=AclLevelField.LEVEL_READ, role=role, count=1,
                       workspace=w)

    def test_040_members_are_joined(self):
        u1 = User()
        u1.email = 'u1'
        u1.save()

        u2 = User()
        u2.email = 'u2'
        u2.save()

        w = Workspace()
        w._user = u1
        w.name = 'workspace'
        w.created_by = u1
        w.save()

        # already materialized roles should be deleted
        Acl.objects.all().delete()
        Role.objects.all().delete()

        m1 = Member()
        m1.workspace = w
        m1.created_by = u1
        m1.user = u1
        m1.save()

        m2 = Member()
        m2.workspace = w
        m2.created_by = u2
        m2.user = u2
        m2.save()

        role = Role()
        role.level = RoleLevelField.LEVEL_READ
        role.created_by = u1
        role.set_object(w)
        role.member = m1
        role.save()

        Member.objects.join_member(m1, m2)

        # 1 acls should be in db, because materialization should happen on
        # regular member
        self.assertEquals(Acl.objects.all().count(), 1)

        # acls should be related to m2
        self.assertEquals(Acl.objects.all()[0].user, m2.user)

    def test_050_update_role(self):
        u = User()
        u.email = 'misan'
        u.save()

        w = Workspace()
        w._user = u
        w.name = 'workspace'
        w.created_by = u
        w.save()

        v = Vault()
        v._user = u
        v.name = 'vault'
        v.workspace = w
        v.created_by = u
        v.save()

        c = Card()
        c.name = 'vault'
        c.vault = v
        c.created_by = u
        c.save()

        m = Member()
        m.user = u
        m.status = MemberStatusField.STATUS_MEMBER
        m.workspace = w
        m.created_by = u
        m.save()

        # already materialized roles should be deleted
        Acl.objects.all().delete()
        Role.objects.all().delete()

        role = Role()
        role.level = RoleLevelField.LEVEL_READ
        role.created_by = u
        role.set_object(v)
        role.member = m
        role.save()

        # parent acls should be read
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, workspace=w)

        role.level = RoleLevelField.LEVEL_WRITE
        role.save()

        # parent acls should be write
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, workspace=w)

        # current should be write
        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=1, vault=v)
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, vault=v)

        # childs should be write
        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=1, card=c)
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, card=c)

    @unittest.skip("should be fixed asap")
    def test_060_create_object(self):
        u = User()
        u.email = 'misan'
        u.save()

        w = Workspace()
        w._user = u
        w.name = 'workspace'
        w.created_by = u
        w.save()

        for a in Acl.objects.all():
            print a
        # current should be write
        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=1, workspace=w)
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, workspace=w)

        v = Vault()
        v._user = u
        v.name = 'vault'
        v.workspace = w
        v.created_by = u
        v.save()

        # current should be write
        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=1, vault=v)
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, vault=v)

    @unittest.skip("should be fixed asap")
    def test_070_delete_object_or_role(self):
        u = User()
        u.email = 'misan'
        u.save()

        w = Workspace()
        w._user = u
        w.name = 'workspace'
        w.created_by = u
        w.save()

        v = Vault()
        v._user = u
        v.name = 'vault'
        v.workspace = w
        v.created_by = u
        v.save()

        # current should be write
        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=1, vault=v)
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, vault=v)

        v.delete()

        # there should be one acls
        self.assertEquals(Acl.objects.count(), 2)

        # there should be one role
        self.assertEquals(Role.objects.count(), 1)

        Role.objects.all().delete()

        # there should be no acls
        self.assertEquals(Acl.objects.count(), 0)

    def test_080_create_role(self):
        u = User()
        u.email = 'misan'
        u.save()

        w = Workspace()
        w._user = u
        w.name = 'workspace'
        w.created_by = u
        w.save()

        v = Vault()
        v._user = u
        v.name = 'vault'
        v.workspace = w
        v.created_by = u
        v.save()

        c = Card()
        c.name = 'vault'
        c.vault = v
        c.created_by = u
        c.save()

        m = Member()
        m.user = u
        m.status = MemberStatusField.STATUS_MEMBER
        m.workspace = w
        m.created_by = u
        m.save()

        # already materialized roles should be deleted
        Acl.objects.all().delete()
        Role.objects.all().delete()

        role = Role()
        role.level = RoleLevelField.LEVEL_CREATE
        role.created_by = u
        role.set_object(v)
        role.member = m
        role.save()

        # workspace read
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, workspace=w)
        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=0, workspace=w)

        # vault read and create
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, vault=v)
        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=0, vault=v)
        self.assertAcl(level=AclLevelField.LEVEL_CREATE, count=1, vault=v)

        # no permissions to card
        self.assertEquals(Role.objects.count(), 1)
        self.assertAcl(count=0, card=c)

        c = Card()
        c.name = 'Another card'
        c.vault = v
        c.created_by = u
        c.save()

        # manage role created to child, read and write on card
        self.assertEquals(Role.objects.count(), 2)
        self.assertAcl(level=AclLevelField.LEVEL_WRITE, count=1, card=c)
        self.assertAcl(level=AclLevelField.LEVEL_CREATE, count=0, card=c)

        a = list(c.acl_set.all())
        self.assertAcl(level=AclLevelField.LEVEL_READ, count=1, card=c)

    def test_080_create_role_multiple_inheritance(self):
        u = User()
        u.email = 'misan'
        u.save()

        w = Workspace()
        w._user = u
        w.name = 'workspace'
        w.created_by = u
        w.save()

        v = Vault()
        v._user = u
        v.name = 'vault'
        v.workspace = w
        v.created_by = u
        v.save()

        m = Member()
        m.user = u
        m.status = MemberStatusField.STATUS_MEMBER
        m.workspace = w
        m.created_by = u
        m.save()

        # already materialized roles should be deleted
        Acl.objects.all().delete()
        Role.objects.all().delete()

        role = Role()
        role.level = RoleLevelField.LEVEL_CREATE
        role.created_by = u
        role.set_object(w)
        role.member = m
        role.save()

        c = Card()
        c.name = 'vault'
        c.vault = v
        c.created_by = u
        c.save()

        # write permission to card because created by user with permission
        # create on workspace
        self.assertEquals(c.role_set.count(), 1)
        self.assertEquals(c.role_set.all()[0].level,
                          RoleLevelField.LEVEL_WRITE)


def acl_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(AclTest))
    return suite



from itertools import ifilter
from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_403_FORBIDDEN, HTTP_400_BAD_REQUEST, \
    HTTP_201_CREATED, HTTP_200_OK
from accounts.business.fields import MemberStatusField
from accounts.models import User, LostKey, Member
from accounts.tests.api import register_api_call, auth_api_call, \
    create_lost_keys_api_call, retrieve_lost_key_api_call, \
    update_lost_key_api_rebuild_call, invite_member_api_call, \
    update_lost_key_api_disable_call
from acls.business.fields import RoleLevelField
from acls.tests.api import create_role_api_call
from libs.version.context import version_context_manager
from vaultier.test.tools import AssertionsMixin
from workspaces.models import Workspace
from workspaces.tests.api import create_workspace_api_call, \
    set_workspace_key_api_call, accept_invitation_api_call


class ApiLostKeyTest(TransactionTestCase, AssertionsMixin):
    """
    Exercise the lost_key API
    """

    def setUp(self):
        version_context_manager.set_enabled(False)

    def create_user(self, email, nickname):
        """
        Creates a new user and return the user instance with the user
        token for authenticated
        :param email: str
        :param nickname: str
        :return:user, token
        """
        register_api_call(email=email, nickname=nickname).data
        user_token = auth_api_call(email=email).data.get('token')
        user = User.objects.get(email=email)
        return user, user_token

    def test_010_should_not_create_resource_with_wrong_email(self):
        """
        Test create view with invalid email
        :return:
        """
        email = 'test010@rclick.com'
        response = create_lost_keys_api_call(email=email)
        self.assertEqual(response.status_code, HTTP_400_BAD_REQUEST,
                         "Email not found ")

    def test_015_hash_should_be_unique(self):
        # create user
        user, token = self.create_user('test015@rclick.com', 'test015')
        # create lost key resource
        create_response = create_lost_keys_api_call(email=user.email)
        # retrieve lost key resource
        lost_key_1 = LostKey.objects.get(pk=create_response.data.get('id'))

        # create second lost key resource
        create_response = create_lost_keys_api_call(email=user.email)
        # retrieve second lost key resource
        lost_key_2 = LostKey.objects.get(pk=create_response.data.get('id'))

        self.assertNotEqual(lost_key_1.hash, lost_key_2.hash,
                            "Different lost keys should contain different "
                            "hash even for the same user")

    def test_020_should_create_a_new_lostkey_resource(self):
        """
        Exercise the creation of a lost key resource
        :return:
        """
        user, token = self.create_user('test020@rclick.com', 'test020')
        response = create_lost_keys_api_call(email=user.email)
        self.assertEqual(response.status_code, HTTP_201_CREATED,
                         "Should find user by email")
        self.assertEqual(response.data.get('email'), user.email,
                         "The response should contain the email")
        msg = "The Api must return the resource id"
        self.assertNotEqual(response.data.get('id'), None, msg)

    def test_030_should_not_retrieve_user_without_hash(self):
        """
        Check authentication base on hash
        :param
        :return:
        """
        user, token = self.create_user('test30@rclick.com', 'test_30')
        # create a new lostkey resource
        response = create_lost_keys_api_call(email=user.email)
        response = retrieve_lost_key_api_call(response.data.get('id'),
                                              'invalid_hash')
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         "Should find the correct user")

    def test_040_should_not_update_user_without_hash(self):
        """
        Check authentication base on hash
        :return:
        """
        user, token = self.create_user('test40@rclick.com', 'test40')
        # create a lost_key resource
        response = create_lost_keys_api_call(email=user.email)
        # update call update without hash
        response = update_lost_key_api_rebuild_call(response.data.get('id'),
                                                    '')
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         "The url needs a hash to perform validation")

    def test_050_should_update_the_user(self):
        """
        Update a user public key
        :return:
        """
        user, token = self.create_user('test050@rclick.com', 'test050')
        new_public_key = 'AnOtHer PuBlIc KeY'
        create_response = create_lost_keys_api_call(email=user.email)
        lost_key = LostKey.objects.get(pk=create_response.data.get('id'))

        response = update_lost_key_api_rebuild_call(lost_key.id, lost_key.hash,
                                                    public_key=new_public_key)
        self.assertEqual(response.status_code, HTTP_200_OK,
                         "The user has been updated")
        lost_key = LostKey.objects.get(pk=lost_key.id)
        self.assertTrue(lost_key.used,
                        "The lost key row must be marked as used")
        user = User.objects.get(email=user.email)
        self.assertEqual(user.public_key, new_public_key,
                         "The user public_key field must be updated")

    @unittest.skip("should be fixed asap")
    def test_060_should_get_the_membership_info(self):
        """
        Create two users and two workspaces invite the second user to the
        workspaces from user one and accept invitation
        Test if the Api returns correctly the memberships for user 1
        :return:
        """
        workspace_key = "Maybe blue, but is the bomb"
        # create users
        user_1, user1token = self.create_user(email='marcos@podebrady.cz',
                                              nickname='marecku')
        user_2, user2token = self.create_user(email='arnoldo@leopoldo.cz',
                                              nickname='clarin')

        # create workspaces for user1
        workspace_1 = create_workspace_api_call(
            token=user1token,
            name='name for the workspace'
        ).data
        workspace_2 = create_workspace_api_call(
            token=user1token,
            name='Very long name'
        ).data

        # approve workspace key for user1
        set_workspace_key_api_call(user1token, 1, workspace_key)

        # invite user 2 to workspace1
        user2member = invite_member_api_call(user1token, user_2.email,
                                             workspace_1.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        create_role_api_call(user1token,
                             user2member.get('id'),
                             to_workspace=workspace_1.get('id'),
                             level=RoleLevelField.LEVEL_READ
                             )

        # accept invitation
        accept_invitation_api_call(user2token, user2member.get('id'),
                                   user2invitation)

        # create lost key resource
        response = create_lost_keys_api_call(email=user_1.email)
        # get the created lost key resource so that we can use the write hash
        lost_key = LostKey.objects.get(pk=response.data.get('id'))

        response = retrieve_lost_key_api_call(lost_key.id, lost_key.hash)
        memberships = response.data.get('memberships')
        # now just the user 1 is the member of both created workspaces
        self.assertTrue(len(memberships) == 2,
                        "Should return the current number of workspaces "
                        "where the user is a member")

        workspaces_fixture = [{'workspace_name': workspace_1.get('name'),
                               'is_recoverable': False},
                              {'workspace_name': workspace_2.get('name'),
                               'is_recoverable': False}]
        index = 0
        for membership in memberships:
            # Workspaces must be shared with an user that has status
            # MemberStatusField.STATUS_MEMBER to be recoverable"
            self.assert_dict(membership, workspaces_fixture[index])
            index += 1

        set_workspace_key_api_call(user1token, user2member.get('id'),
                                   workspace_key)

        response = retrieve_lost_key_api_call(lost_key.id, lost_key.hash)

        memberships = response.data.get('memberships')
        self.assertTrue(len(memberships) == 2,
                        "Should return the current number of workspaces "
                        "where the user is a member"
                        )
        list(Member.objects.filter(workspace_id=workspace_1.get('id')))

        workspaces_fixture = [{'workspace_name': workspace_1.get('name'),
                               'is_recoverable': True},
                              {'workspace_name': workspace_2.get('name'),
                               'is_recoverable': False}]
        index = 0
        for membership in memberships:
            # The API can count individually the members for each workspace
            # and differentiate by status
            self.assert_dict(membership, workspaces_fixture[index])
            index += 1

        update_lost_key_api_rebuild_call(lost_key.id,
                                         lost_key.hash,
                                         public_key='Some new key')

        user1_total_memberships_different_200 = \
            Member.objects.filter(
                user=lost_key.created_by
            ).exclude(
                status=MemberStatusField.STATUS_MEMBER_WITHOUT_WORKSPACE_KEY
            ).count()
        msg = "There should not be any membership with status different than "\
              "MemberStatusField.STATUS_MEMBER_WITHOUT_WORKSPACE_KEY"

        self.assertEqual(user1_total_memberships_different_200, 1, msg)

    def test_065_membership_should_not_contain_duplicate_workspaces(self):
        # create users
        user_1, user1token = self.create_user(email='test65@rclick.cz',
                                              nickname='user_1')
        self.create_user(email='test065@rclick.cz', nickname='user_2')

        # create workspaces for user1
        workspace_1 = create_workspace_api_call(token=user1token,
                                                name='workspace_1_name').data
        workspace_2 = create_workspace_api_call(token=user1token,
                                                name='workspace_2_name').data
        workspace_3 = create_workspace_api_call(token=user1token,
                                                name='workspace_3_name').data
        workspace_4 = create_workspace_api_call(token=user1token,
                                                name='workspace_4_name').data

        # create lost key resource
        response = create_lost_keys_api_call(email=user_1.email)
        # get the created lost key resource so that we can use the write hash
        lost_key = LostKey.objects.get(pk=response.data.get('id'))

        response = retrieve_lost_key_api_call(lost_key.id, lost_key.hash)
        memberships = response.data.get('memberships')

        self.assertEqual(len(memberships), 4,
                         "Membership contain the wright number of workspaces")

        msg = "The name cof the workspace can not be duplicate unless " \
              "two workspaces contain the same name"
        self.assertEqual(
            len(list(
                ifilter(
                    lambda workspace: workspace_1.get('name') == workspace.get(
                        'workspace_name'), memberships))), 1, msg)
        self.assertEqual(
            len(list(
                ifilter(
                    lambda workspace: workspace_3.get('name') == workspace.get(
                        'workspace_name'), memberships))), 1, msg)
        self.assertEqual(
            len(list(
                ifilter(
                    lambda workspace: workspace_2.get('name') == workspace.get(
                        'workspace_name'), memberships))), 1, msg)
        self.assertEqual(
            len(list(
                ifilter(
                    lambda workspace: workspace_4.get('name') == workspace.get(
                        'workspace_name'), memberships))), 1, msg)

        # now every workspace is not recoverable
        self.assertEqual(
            len(list(
                ifilter(lambda workspace: workspace.get('is_recoverable'),
                        memberships))), 0,
            "Non shared workspaces are unrecoverable")

    def test_070_cannot_use_lost_key_resource_twice(self):
        # create a lost key resource
        user, token = self.create_user('test70@email.com', 'test_70')
        create_response = create_lost_keys_api_call(user.email)
        # retrieve the lost_key resource because the hash is needed
        lost_key = LostKey.objects.get(pk=create_response.data.get('id'))

        response = update_lost_key_api_rebuild_call(
            lost_key.id, lost_key.hash, 'Some changes on the public key'
        )

        self.assertEqual(response.status_code, HTTP_200_OK,
                         "the resource was correctly updated")

        response = update_lost_key_api_rebuild_call(
            lost_key.id, lost_key.hash, 'Random changes on the public key'
        )

        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         "Once used the resource can not be used again")

    def test_080_should_disable_workspaces(self):
        workspace_key = '!-@#$%.*'
        # create users
        user_1, user1token = self.create_user(email='test80.1@rclick.cz',
                                              nickname='first_user')
        user_2, user2token = self.create_user(email='test80.2@rclick.cz',
                                              nickname='second_user')

        # create workspaces for user1
        workspace_1 = create_workspace_api_call(token=user1token,
                                                name='first workspace').data
        workspace_2 = create_workspace_api_call(
            token=user1token,
            name='second workspace').data

        # approve workspace key for user1
        set_workspace_key_api_call(user1token, 8, workspace_key)
        # invite user 2 to user 1 workspace1
        user2member = invite_member_api_call(token=user1token,
                                             email=user_2.email,
                                             workspace=workspace_1.get('id')
                                             ).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        create_role_api_call(user1token,
                             user2member.get('id'),
                             to_workspace=workspace_1.get('id'),
                             level=RoleLevelField.LEVEL_READ
                             )
        # accept invitation
        accept_invitation_api_call(user2token,
                                   user2member.get('id'),
                                   user2invitation
                                   )
        # create lost key resource
        response = create_lost_keys_api_call(email=user_1.email)
        # get the created lost key resource so that we can use the write hash
        lost_key = LostKey.objects.get(pk=response.data.get('id'))

        workspaces = Workspace.objects.filter(membership__user=user_1)
        self.assertEquals(workspaces.count(), 2)

        # disable key
        update_lost_key_api_disable_call(lost_key.id,
                                         lost_key.hash,
                                         public_key='-'
                                         )

        # get workspaces
        active_workspace = Workspace.objects.filter(membership__user=user_1)
        self.assertEquals(active_workspace.count(), 1)

        all_workspaces = Workspace.objects.include_deleted().filter(
            membership__user=user_1)
        self.assertEquals(all_workspaces.count(), 2)

        broken_status_workspaces = Workspace.objects.include_deleted().\
            filter(membership__user=user_1,
                   membership__status=MemberStatusField.STATUS_MEMBER_BROKEN
                   )
        self.assertEquals(broken_status_workspaces.count(),
                          all_workspaces.count())

    @unittest.skip("should be fixed asap")
    def test_090_should_disable_just_unrecoverable_workspaces(self):
        workspace_key = '!-@#$%.*'
        # create users
        user_1, user1token = self.create_user(email='test90.1@rclick.cz',
                                              nickname='first_user')
        user_2, user2token = self.create_user(email='test90.2@rclick.cz',
                                              nickname='second_user')

        # create workspaces for user1
        workspace_1 = create_workspace_api_call(token=user1token,
                                                name='first workspace').data
        # approve workspace key for user1
        set_workspace_key_api_call(user1token, 11, workspace_key)
        # invite user 2 to user 1 workspace1
        user2member = invite_member_api_call(token=user1token,
                                             email=user_2.email,
                                             workspace=workspace_1.get(
                                                 'id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        create_role_api_call(user1token,
                             user2member.get('id'),
                             to_workspace=workspace_1.get('id'),
                             level=RoleLevelField.LEVEL_READ
                             )
        # accept invitation
        accept_invitation_api_call(user2token,
                                   user2member.get('id'),
                                   user2invitation
                                   )
        # create lost key resource
        response = create_lost_keys_api_call(email=user_1.email)
        # get the created lost key resource so that we can use the write hash
        lost_key = LostKey.objects.get(pk=response.data.get('id'))

        workspaces = Workspace.objects.filter(membership__user=user_1)
        self.assertEquals(workspaces.count(), 2,
                          "We should be able to read every workspace")

        # disable key
        update_lost_key_api_rebuild_call(lost_key.id,
                                         lost_key.hash,
                                         public_key='Meine neu public key'
                                         )

        # get workspaces
        active_workspace = Workspace.objects.filter(
            membership__user=user_1)
        self.assertEquals(active_workspace.count(), 1,
                          "Unshared workspace was deleted")

        all_workspaces = Workspace.objects.include_deleted().filter(
            membership__user=user_1)
        self.assertEquals(all_workspaces.count(), 2,
                          "The workspace where just soft deleted")

        broken_status_workspaces = Workspace.objects \
            .include_deleted().filter(
                membership__user=user_1,
                membership__status=MemberStatusField.STATUS_MEMBER_BROKEN
            )
        self.assertTrue(
            broken_status_workspaces.count() < all_workspaces.count(),
            "Shared workspaces has status MembershipStatusField."
            "STATUS_MEMBER_WITHOUT_WORKSPACE_KEY")


def lost_keys_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ApiLostKeyTest))
    return suite



from django.core.management.base import BaseCommand
from accounts.business.fields import MemberStatusField

from acls.models import Acl, Role
from acls.business.perms.materializer import CreateRoleMaterializer


class Command(BaseCommand):

    def handle(self, *command_args, **command_options):
        Acl.objects.all().delete()
        for role in Role.objects.all():
            if not role.member.status == MemberStatusField.STATUS_INVITED:
                m = CreateRoleMaterializer(role)
                m.materialize(role.get_object())



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from rest_framework.status import HTTP_200_OK, HTTP_403_FORBIDDEN, \
    HTTP_204_NO_CONTENT, HTTP_404_NOT_FOUND
from accounts.business.fields import MemberStatusField
from accounts.models import Member
from accounts.tests.api import register_api_call, auth_api_call, \
    invite_member_api_call, delete_member_api_call, list_members_api_call
from acls.business.fields import RoleLevelField
from acls.models import Role
from acls.tests.api import create_role_api_call
from libs.version.context import version_context_manager
from vaultier.test.tools import format_response
from workspaces.tests.api import create_workspace_api_call, \
    accept_invitation_api_call, retrieve_invitation_api_call


class ApiInviteTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_010_invitations_permissions_to_workspace(self):
        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'tomas@rclick.cz'
        register_api_call(email=email, nickname='tomas').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(user1token,
                                              name='workspace1').data

        # invite anonymous forbidden
        response = invite_member_api_call(None, email='jakub@rclick.cz',
                                          workspace=workspace.get('id'))
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

        # user2 tries to invite should be forbidden
        response = invite_member_api_call(user2token, email='jakub@rclick.cz',
                                          workspace=workspace.get('id'))
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

        # user1 tries to invite user2
        response = invite_member_api_call(user1token, email='jakub@rclick.cz',
                                          workspace=workspace.get('id'))
        self.assertEqual(response.status_code, HTTP_200_OK,
                         format_response(response))

        # user1 tries to invite user2 again
        response = invite_member_api_call(user1token, email='jakub@rclick.cz',
                                          workspace=workspace.get('id'))
        self.assertEqual(response.status_code, HTTP_200_OK,
                         format_response(response))

        # event when invited same user twice, only one member should exist
        self.assertEqual(Member.objects.all().count(), 2)

        # user2 creates new workspace, user1 should be forbidden to
        # invitate to workspace2
        workspace2 = create_workspace_api_call(user2token,
                                               name='workspace2').data
        response = invite_member_api_call(user1token, email='jakub@rclick.cz',
                                          workspace=workspace2.get('id'))
        self.assertEqual(response.status_code, HTTP_403_FORBIDDEN,
                         format_response(response))

    def test_020_acceptations_membership_merging_and_accepting(self):
        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'jakub@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace and invites 3 users
        user1workspace = create_workspace_api_call(user1token,
                                                   name='workspace').data
        # 1 role in db for exisitng workspace
        self.assertEqual(Role.objects.all().count(), 1)

        user2member = invite_member_api_call(user1token, 'jakub@rclick.cz',
                                             user1workspace.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash

        # create role for member 2
        create_role_api_call(
            user1token,
            user2member.get('id'),
            level=RoleLevelField.LEVEL_READ,
            to_workspace=user1workspace.get('id')
        )

        user3member = invite_member_api_call(user1token, 'stepan@rclick.cz',
                                             user1workspace.get('id')).data
        user3invitation = Member.objects.get(
            pk=user3member.get('id')).invitation_hash

        # create role for member 3
        create_role_api_call(
            user1token,
            user3member.get('id'),
            level=RoleLevelField.LEVEL_WRITE,
            to_workspace=user1workspace.get('id')
        )

        # 3 invitations in db, one approved member
        self.assertEqual(Member.objects.filter(
            status=MemberStatusField.STATUS_MEMBER_WITHOUT_WORKSPACE_KEY
        ).count(), 1)

        self.assertEqual(Member.objects.filter(
            status=MemberStatusField.STATUS_INVITED).count(), 2)

        # 3 roles in db, 2 for invitations, 1 for exisitng workspace
        self.assertEqual(Role.objects.all().count(), 3)

        # user2 will accept all invitations, only one member should exists
        response = accept_invitation_api_call(user2token,
                                              user2member.get('id'),
                                              user2invitation)
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # 0 approved member, 2 non approved member
        self.assertEqual(
            Member.objects.filter(
                status=MemberStatusField.STATUS_MEMBER_WITHOUT_WORKSPACE_KEY
            ).count(), 2)

        # 3 roles only should exists, for user1 and for user2 and
        # for invitation 3
        self.assertEqual(Role.objects.all().count(), 3)

        response = accept_invitation_api_call(user2token,
                                              user3member.get('id'),
                                              user3invitation)
        self.assertEqual(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # 2 roles only should exists, for user1 and for user2
        self.assertEqual(Role.objects.all().count(), 2)

        # both roles should be write
        self.assertEqual(
            Role.objects.filter(level=RoleLevelField.LEVEL_WRITE).count(), 2)

    def test_030_list_roles_for_hash(self):
        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'jakub@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user2token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        user1workspace = create_workspace_api_call(user1token,
                                                   name='u1_workspace').data
        self.assertEqual(Role.objects.all().count(), 1)

        # invite user 2
        user2member = invite_member_api_call(user1token, 'jakub@rclick.cz',
                                             user1workspace.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        create_role_api_call(user1token, user2member.get('id'),
                             to_workspace=user1workspace.get('id'),
                             level=RoleLevelField.LEVEL_READ
                             )

        # invite user 3
        user3member = invite_member_api_call(user1token, 'marcel@rclick.cz',
                                             user1workspace.get('id')
                                             ).data
        create_role_api_call(user1token, user3member.get('id'),
                             to_workspace=user1workspace.get('id'),
                             level=RoleLevelField.LEVEL_READ)

        # list roles for workspace1 without hash, should be forbidden
        response = retrieve_invitation_api_call(user2token,
                                                user2member.get('id'),
                                                'testovaci-hash')
        self.assertEquals(
            response.status_code,
            HTTP_404_NOT_FOUND,
            format_response(response)
        )

        # list roles for workspace1 with hash, should be allowed
        response = retrieve_invitation_api_call(user2token,
                                                user2member.get('id'),
                                                user2invitation)
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # only one role should be visible
        self.assertEquals(
            len(response.data.get('roles')),
            1,
            format_response(response)
        )

    def test_050_delete_member(self):
        # only workspace manager can delete member
        # no roles should be in db for member
        # no acl should be in db for member

        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'jakub@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user2token = auth_api_call(email=email).data.get('token')

        # create third user
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user3token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        user1workspace = create_workspace_api_call(user1token,
                                                   name='u1_workspace').data
        self.assertEqual(Role.objects.all().count(), 1)

        # invite user 2
        user2member = invite_member_api_call(user1token, 'jakub@rclick.cz',
                                             user1workspace.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')).invitation_hash
        create_role_api_call(user1token,
                             user2member.get('id'),
                             to_workspace=user1workspace.get('id'),
                             level=RoleLevelField.LEVEL_READ)
        response = accept_invitation_api_call(user2token,
                                              user2member.get('id'),
                                              user2invitation)
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        # invite user 3
        user3member = invite_member_api_call(user1token, 'marcel@rclick.cz',
                                             user1workspace.get('id')).data
        user3invitation = Member.objects.get(
            pk=user3member.get('id')).invitation_hash
        create_role_api_call(user1token,
                             user3member.get('id'),
                             to_workspace=user1workspace.get('id'),
                             level=RoleLevelField.LEVEL_READ
                             )
        response = accept_invitation_api_call(user3token,
                                              user3member.get('id'),
                                              user3invitation)
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )

        response = delete_member_api_call(user2token, user3member.get('id'))
        self.assertEquals(
            response.status_code,
            HTTP_403_FORBIDDEN,
            format_response(response)
        )

        response = delete_member_api_call(user1token, user3member.get('id'))
        self.assertEquals(
            response.status_code,
            HTTP_204_NO_CONTENT,
            format_response(response)
        )

    def test_60_list_members(self):
        # only workspace manager can list members
        # manager should se members only of workspaces he is manager of

        # create first user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='jan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create second user
        email = 'jakub@rclick.cz'
        register_api_call(email=email, nickname='jakub').data
        user2token = auth_api_call(email=email).data.get('token')

        # create third  user
        email = 'marcel@rclick.cz'
        register_api_call(email=email, nickname='marcel').data
        user3token = auth_api_call(email=email).data.get('token')

        # create workspace for user1
        user1workspace = create_workspace_api_call(user1token,
                                                   name='u1_workspace').data
        self.assertEqual(Role.objects.all().count(), 1)

        # invite user 2
        user2member = invite_member_api_call(user1token, 'jakub@rclick.cz',
                                             user1workspace.get('id')).data
        user2invitation = Member.objects.get(
            pk=user2member.get('id')
        ).invitation_hash

        create_role_api_call(user1token,
                             user2member.get('id'),
                             to_workspace=user1workspace.get('id'),
                             level=RoleLevelField.LEVEL_READ
                             )
        accept_invitation_api_call(user2token,
                                   user2member.get('id'),
                                   user2invitation
                                   )

        response = list_members_api_call(user1token, user1workspace.get('id'))
        self.assertEquals(
            response.status_code,
            HTTP_200_OK,
            format_response(response)
        )
        self.assertEquals(
            len(response.data),
            2,
            format_response(response)
        )
        response = list_members_api_call(user3token, user1workspace.get('id'))
        self.assertEquals(
            len(response.data),
            0,
            format_response(response)
        )


def member_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(ApiInviteTest))
    return suite



from django.core.urlresolvers import reverse

from vaultier.test.tools import VaultierAPIClient


def create_role_api_call(token, member, to_workspace=None, to_vault=None,
                         to_card=None, level=None):
    url = reverse('role-list')
    client = VaultierAPIClient()
    client.token(token)

    data = {
        'member': member,
        'level': level
    }

    if to_workspace:
        data['to_workspace'] = to_workspace
    if to_vault:
        data['to_vault'] = to_vault
    if to_card:
        data['to_card'] = to_card

    response = client.post(url, data)
    return response


def update_role_api_call(token, id, level):
    url = reverse('role-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)

    data = {
        'level': level
    }

    response = client.put(url, data)
    return response


def delete_role_api_call(token, id):
    url = reverse('role-detail', args=(id,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.delete(url)
    return response


def list_role_api_call(token, to_workspace=None, to_vault=None, to_card=None):
    url = reverse('role-list')
    client = VaultierAPIClient()
    client.token(token)

    data = {}

    if to_workspace:
        data['to_workspace'] = to_workspace
    if to_vault:
        data['to_vault'] = to_vault
    if to_card:
        data['to_card'] = to_card

    response = client.get(url, data)
    return response



from django.conf import settings
from django.core.mail.message import EmailMultiAlternatives
from django.template.context import Context
from django.template.loader import get_template
from pynliner import Pynliner
from celery.contrib.methods import task_method
from celery import current_app as app


class VaultierMailer(object):
    """
    Vaultier base mailer class. Feel free to override me

    Base usage:
    >>> mailer = VaultierMailer(template='mailer/my_template',
    >>>                         subject='[Vaultier] New folder')
    >>> mailer.add_to('john@doe.org').send()

    Note: pass template name without extension. Plaintext and html
    version should exists side by side
    """

    _to = []
    from_email = None
    subject = None
    template_style = 'mailer/layout.css'
    object = None
    _template = 'mailer/layout'

    def __init__(self, **kwargs):
        self._to = kwargs.get('to', [])
        self.from_email = kwargs.get('from_email',
                                     settings.VAULTIER.get('from_email'))
        self.subject = kwargs.get('subject', self.subject)
        self._template = kwargs.get('template', self._template)
        self.template_style = kwargs.get('template_style', self.template_style)
        self.object = kwargs.get('object')

    @property
    def to(self):
        """
        Return list of email recipients
        """
        try:
            return iter(self._to)
        except TypeError:
            return [self._to]

    def add_to(self, value):
        """
        Add recipient
        """
        if value not in self._to:
            self._to.append(value)
        return self

    @property
    def template_plain(self):
        """
        Return Template instance for plaintext mail

        :return: Template
        """

        return get_template("{}.txt".format(self._template))

    @property
    def template_html(self):
        """
        Return Template instance for plaintext mail

        :return: Template
        """
        return get_template("{}.html".format(self._template))

    def send(self, **kwargs):
        """
        Send an email, base on the template and context given to the address
        in recipients.

        :return: None
        """
        if kwargs.pop('async', True):
            self._send.delay(**kwargs)
        else:
            self._send(**kwargs)

    def format_subject(self):
        """
        Hook to format, compile or whatever with subject before send

        :return: string
        """
        return self.subject

    @app.task(filter=task_method)
    def _send(self, **kwargs):
        """
        Actually send email in background

        :return:
        """
        context = self._build_context(**kwargs)
        plain, html = self._render(context)
        msg = EmailMultiAlternatives(
            from_email=self.from_email,
            to=self.to,
            body=plain,
            subject=self.format_subject()
        )

        msg.attach_alternative(html, 'text/html')
        msg.send()

    def _render(self, context):
        """
        Renders the plain and html versions of a template.
        Return both in a tuple, where the first element is the plain text
        version and the second element is the html version
        :return: (str, str,)
        """
        if not context:
            context = Context({})

        plain = self.template_plain.render(context)
        html = self.template_html.render(context)
        css = get_template(self.template_style).render(Context({}))

        p = Pynliner()
        html = p.from_string(html).with_cssString(css).run()

        return plain, html

    def _build_context(self, **kwargs):
        """
        Must return an instance of django.template.context.Context
        with the template variables set for each implementation
        :return: None
        """

        return Context(kwargs)



from rest_framework.permissions import BasePermission
from accounts.models import User


class CanManageUserPermission(BasePermission):

    def has_permission(self, request, view):
        if view.action == 'create':
            return True
        if request.user and request.user.is_authenticated():
            return True
        return False

    def has_object_permission(self, request, view, obj):
        user = request.user
        return user.id == obj.id


class LostKeyPermission(BasePermission):
    """
    Custom permission class based on view
    """

    def has_object_permission(self, request, view, obj):
        if view.action == 'create':
            return True
        else:
            return isinstance(request.user, User)

    def has_permission(self, request, view):
        if view.action == 'create':
            return True
        else:
            return isinstance(request.user, User)



from user import test_suite
from .lostkey import lost_keys_suite
from .member import member_suite
from django.utils.unittest.suite import TestSuite


def suite():
    s = TestSuite()
    # s.addTest(test_suite())
    s.addTest(lost_keys_suite())
    # s.addTest(member_suite())
    return s



from .acl import acl_suite
from .role import role_suite
from django.utils.unittest.suite import TestSuite


def suite():
    s = TestSuite()
    s.addTest(role_suite())
    s.addTest(acl_suite())
    return s



from django.dispatch import Signal

post_change = Signal()
"""
Signal sent whenever an instance is saved or deleted
and changes have been recorded.

signal receiver should be following:
def on_change(
    sender=None,
    instance=None,
    overwritten_values=None,
    event_type=None,
    **kwargs
)
"""


from django.db.models import signals

INSERT = 10
UPDATE = 20
DELETE = 30
SOFT_DELETE = 40


class ChangesMixin(object):
    """
    Mixin could be used on models to get access to model lifecycle.
    Generally mixin provides:
     - dirtiness
     - access to overwritten values by latest save/delete call
     - signal when change happen
     - hooks on model to simplify code base

    Example model to use ChangesMixin
        class Garage(ChangesMixin, models.Model):
            class Meta:
                db_table = u'garage'

            car1 = models.CharField(max_length=255, default='')
            car2 = models.CharField(max_length=255, default='')

            def on_post_change(self, sender=None,instance=None,
                    overwritten_values=None,event_type=None,**kwargs):
                raise 'this is called when change happen'

            def on_pre_save(self, sender=None,instance=None**kwargs):
                raise 'this is called when pre_save happen'

     Example to for connect signal

        def on_change(self, sender=None,instance=None,overwritten_values=None,
                      event_type=None,**kwargs):
            // do something here
            pass

        post_change.connect(on_change, sender=Garage)

    Description of lifecycle
        self.dirty_values()  - return dict of clean values before changed
            locally
        self.previous_values() - return dict of values before latest safe
        self.clean_values() - return dict of values when new model was created
            or persisted model hydrated
        self.current_values() - return dict of current values


    """
    _overwritten_values = {}
    _previous_values = {}
    _clean_values = {}
    _post_change_signal_disabled = 0

    def __init__(self, *args, **kwargs):
        super(ChangesMixin, self).__init__(*args, **kwargs)

        if self.pk is None:
            # new
            self._clean_values = self._current_values()
        else:
            # existing
            self._clean_values = self._current_values()
            self._save_state()

        def _post_save(sender, instance, **kwargs):
            instance._post_save(**kwargs)

        def _post_delete(sender, instance, **kwargs):
            instance._post_delete(**kwargs)

        def _pre_save(sender, instance, **kwargs):
            try:
                getattr(instance, 'on_pre_save')
                exist = True
            except:
                exist = False
            if exist:
                instance.on_pre_save(sender=sender, instance=instance,
                                     **kwargs)

        signals.pre_save.connect(
            _pre_save,
            weak=False,
            sender=self.__class__,
            dispatch_uid='django-changes-pre-save-{}'.format(
                self.__class__.__name__)
        )

        signals.post_save.connect(
            _post_save,
            weak=False,
            sender=self.__class__,
            dispatch_uid='django-changes-post-save-{}'.format(
                self.__class__.__name__)
        )
        signals.post_delete.connect(
            _post_delete,
            weak=False,
            sender=self.__class__,
            dispatch_uid='django-changes-post-delete-{}'.format(
                self.__class__.__name__)
        )

    def _compute_changed_fields(self, current, previous):
        result = {}
        for key in current.keys():
            if key in previous and previous[key] is not current[key]:
                result[key] = previous[key]

        return result

    def _save_state(self):
        self._previous_values = self._clean_values
        self._clean_values = self._current_values()
        self._overwritten_values = self._compute_changed_fields(
            self._clean_values, self._previous_values)

    def _current_values(self):
        """
        Returns a ``field -> value`` dict of the current state of the instance.
        """
        field_names = set()
        [field_names.add(f.name) for f in self._meta.local_fields]
        [field_names.add(f.attname) for f in self._meta.local_fields]

        dict = {}
        for field_name in field_names:
            try:
                dict[field_name] = getattr(self, field_name)
            except:
                pass

        return dict

    def _post_delete(self, **kwargs):
        overwritten_values = self._current_values()
        self._save_state()

        if self._post_change_signal_disabled == 0:
            post_change.send(
                sender=self.__class__,
                instance=self,
                event_type=DELETE,
                overwritten_values=overwritten_values,
            )

    def _fire_post_change(self, sender=None, instance=None, event_type=None,
                          overwritten_values=None):
        exist = False
        try:
            getattr(self, 'on_post_change')
            exist = True
        except:
            exist = False

        if exist:
            self.on_post_change(
                sender=sender,
                instance=instance,
                event_type=event_type,
                overwritten_values=overwritten_values
            )

        post_change.send(
            sender=sender,
            instance=instance,
            event_type=event_type,
            overwritten_values=overwritten_values
        )

    def _post_save(self, **kwargs):
        self._save_state()

        if self._post_change_signal_disabled == 0:

            if kwargs.get('created'):
                event_type = INSERT
            else:
                event_type = UPDATE

            self._fire_post_change(
                sender=self.__class__,
                instance=self,
                event_type=event_type,
                overwritten_values=self._overwritten_values
            )

    def dirty_values(self):
        return self._compute_changed_fields(self._current_values(),
                                            self._clean_values)

    def overwritten_values(self):
        return self._overwritten_values

    def previous_values(self):
        return self._previous_values

    def clean_values(self):
        return self._clean_values

    def set_post_change_signal_enabled(self, enabled):
        if enabled:
            toadd = -1
        else:
            toadd = 1

        self._post_change_signal_disabled += toadd

        if self._post_change_signal_disabled < 0:
            self._post_change_signal_disabled = 0



from django.db import models
from libs.changes.changes import ChangesMixin
from libs.softdelete.softdelete import SoftDeleteMixin
from libs.tree.iterator import TreeIterableModelMixin
from django.db.models.deletion import PROTECT
from .business.tree import WorkspaceTreeIterator
from .business.managers import WorkspaceManager


class Workspace(ChangesMixin, SoftDeleteMixin, TreeIterableModelMixin,
                models.Model):
    tree_iterator_class = WorkspaceTreeIterator

    name = models.CharField(max_length=255)

    slug = models.CharField(max_length=255, default='')
    description = models.CharField(max_length=1024, blank=True, null=True)
    created_by = models.ForeignKey('accounts.User', on_delete=PROTECT)
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)

    objects = WorkspaceManager()

    class Meta:
        db_table = u'vaultier_workspace'

    def __str__(self):
        return self.name

    def __unicode__(self):
        return unicode(str(self))

    def save(self, *args, **kwargs):
        super(Workspace, self).save(*args, **kwargs)
        if kwargs.get('force_insert', False):
            Workspace.objects.create_member_with_workspace(self)



from django.core.urlresolvers import reverse

from vaultier.test.tools import VaultierAPIClient


def accept_invitation_api_call(token, id=None, hash=None):
    url = reverse('invitation-detail', args=(hash,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.put(url)
    return response


def retrieve_invitation_api_call(token, member, hash):
    url = reverse('invitation-detail', args=(hash,))
    client = VaultierAPIClient()
    client.token(token)
    response = client.get(url)
    return response



from rest_framework.permissions import BasePermission
from acls.business.fields import AclLevelField
from acls.business.permissions import has_object_acl


class CanManageSecretPermission(BasePermission):

    def has_object_permission(self, request, view, obj):

        if view.action == 'retrieve' or view.action == 'list':
            return has_object_acl(request.user, obj.card,
                                  AclLevelField.LEVEL_READ)
        else:
            # check permission to card
            return has_object_acl(request.user, obj.card,
                                  AclLevelField.LEVEL_WRITE)



# -*- coding: utf-8 -*-
from south.utils import datetime_utils as datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    depends_on = (('vaults', '0001_initial'), ('cards', '0001_initial'))

    def forwards(self, orm):
        # Adding model 'Acl'
        db.create_table(u'vaultier_acl', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('vaultier.business.fields.ObjectReferenceTypeField')()),
            ('to_workspace', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['workspaces.Workspace'], null=True, blank=True)),
            ('to_vault', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['vaults.Vault'], null=True, blank=True)),
            ('to_card', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cards.Card'], null=True, blank=True)),
            ('level', self.gf('acls.business.fields.RoleLevelField')()),
            ('direction', self.gf('acls.business.fields.AclDirectionField')()),
            ('role', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['acls.Role'])),
            ('user', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['accounts.User'])),
        ))
        db.send_create_signal(u'acls', ['Acl'])

        # Adding model 'Role'
        db.create_table(u'vaultier_role', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('type', self.gf('vaultier.business.fields.ObjectReferenceTypeField')()),
            ('to_workspace', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['workspaces.Workspace'], null=True, blank=True)),
            ('to_vault', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['vaults.Vault'], null=True, blank=True)),
            ('to_card', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['cards.Card'], null=True, blank=True)),
            ('member', self.gf('django.db.models.fields.related.ForeignKey')(to=orm['accounts.Member'])),
            ('level', self.gf('acls.business.fields.RoleLevelField')()),
            ('created_at', self.gf('django.db.models.fields.DateTimeField')(auto_now_add=True, blank=True)),
            ('updated_at', self.gf('django.db.models.fields.DateTimeField')(auto_now=True, blank=True)),
            ('created_by', self.gf('django.db.models.fields.related.ForeignKey')(related_name='roles_created', on_delete=models.PROTECT, to=orm['accounts.User'])),
        ))
        db.send_create_signal(u'acls', ['Role'])


    def backwards(self, orm):
        # Deleting model 'Acl'
        db.delete_table(u'vaultier_acl')

        # Deleting model 'Role'
        db.delete_table(u'vaultier_role')


    models = {
        u'accounts.member': {
            'Meta': {'object_name': 'Member', 'db_table': "u'vaultier_member'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'members_created'", 'on_delete': 'models.PROTECT', 'to': u"orm['accounts.User']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'invitation_email': ('libs.lowercasefield.lowercasefield.LowerCaseCharField', [], {'max_length': '1024', 'null': 'True'}),
            'invitation_hash': ('django.db.models.fields.CharField', [], {'max_length': '64', 'unique': 'True', 'null': 'True'}),
            'status': ('accounts.business.fields.MemberStatusField', [], {'default': '3'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'membership'", 'null': 'True', 'to': u"orm['accounts.User']"}),
            'workspace': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'membership'", 'to': u"orm['workspaces.Workspace']"}),
            'workspace_key': ('django.db.models.fields.CharField', [], {'max_length': '4096'})
        },
        u'accounts.user': {
            'Meta': {'object_name': 'User', 'db_table': "u'vaultier_user'"},
            'email': ('libs.lowercasefield.lowercasefield.LowerCaseCharField', [], {'unique': 'True', 'max_length': '255'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'nickname': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'public_key': ('django.db.models.fields.CharField', [], {'max_length': '1024'})
        },
        u'acls.acl': {
            'Meta': {'object_name': 'Acl', 'db_table': "u'vaultier_acl'"},
            'direction': ('acls.business.fields.AclDirectionField', [], {}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('acls.business.fields.RoleLevelField', [], {}),
            'role': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['acls.Role']"}),
            'to_card': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['cards.Card']", 'null': 'True', 'blank': 'True'}),
            'to_vault': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['vaults.Vault']", 'null': 'True', 'blank': 'True'}),
            'to_workspace': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['workspaces.Workspace']", 'null': 'True', 'blank': 'True'}),
            'type': ('vaultier.business.fields.ObjectReferenceTypeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']"})
        },
        u'acls.role': {
            'Meta': {'object_name': 'Role', 'db_table': "u'vaultier_role'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'roles_created'", 'on_delete': 'models.PROTECT', 'to': u"orm['accounts.User']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'level': ('acls.business.fields.RoleLevelField', [], {}),
            'member': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.Member']"}),
            'to_card': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['cards.Card']", 'null': 'True', 'blank': 'True'}),
            'to_vault': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['vaults.Vault']", 'null': 'True', 'blank': 'True'}),
            'to_workspace': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['workspaces.Workspace']", 'null': 'True', 'blank': 'True'}),
            'type': ('vaultier.business.fields.ObjectReferenceTypeField', [], {}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        },
        u'cards.card': {
            'Meta': {'object_name': 'Card', 'db_table': "u'vaultier_card'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']", 'on_delete': 'models.PROTECT'}),
            'deleted_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'vault': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['vaults.Vault']"})
        },
        u'vaults.vault': {
            'Meta': {'object_name': 'Vault', 'db_table': "u'vaultier_vault'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']", 'on_delete': 'models.PROTECT'}),
            'deleted_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'}),
            'workspace': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['workspaces.Workspace']"})
        },
        u'workspaces.workspace': {
            'Meta': {'object_name': 'Workspace', 'db_table': "u'vaultier_workspace'"},
            'created_at': ('django.db.models.fields.DateTimeField', [], {'auto_now_add': 'True', 'blank': 'True'}),
            'created_by': ('django.db.models.fields.related.ForeignKey', [], {'to': u"orm['accounts.User']", 'on_delete': 'models.PROTECT'}),
            'deleted_at': ('django.db.models.fields.DateTimeField', [], {'null': 'True'}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '1024', 'null': 'True', 'blank': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'}),
            'slug': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '255'}),
            'updated_at': ('django.db.models.fields.DateTimeField', [], {'auto_now': 'True', 'blank': 'True'})
        }
    }

    complete_apps = ['acls']


from libs.tree.iterator import AbstractTreeIterator


class VaultTreeIterator(AbstractTreeIterator):

    def get_parent_object(self):
        return self.get_object().workspace

    def get_child_objects(self):
        return self.get_object().card_set.all()



from django.test.testcases import TransactionTestCase
from django.utils import unittest
from django.utils.unittest.suite import TestSuite
from accounts.tests.api import register_api_call, auth_api_call
from libs.version.context import version_context_manager
from workspaces.models import Workspace
from workspaces.tests.api import create_workspace_api_call, \
    delete_workspace_api_call


class VaultSoftDeleteTest(TransactionTestCase):

    def setUp(self):
        version_context_manager.set_enabled(False)

    def test_010_softdelete(self):

        # create user
        email = 'jan@rclick.cz'
        register_api_call(email=email, nickname='Misan').data
        user1token = auth_api_call(email=email).data.get('token')

        # create workspace
        workspace = create_workspace_api_call(
            user1token, name='workspace').data

        delete_workspace_api_call(user1token, workspace.get('id'))

        workspaces = Workspace.objects.filter(id=workspace.get('id'))
        self.assertEquals(workspaces.count(), 0)

        workspaces = Workspace.objects.include_deleted().filter(
            id=workspace.get('id'))
        self.assertEquals(workspaces.count(), 1)


def workspace_softdelete_suite():
    suite = TestSuite()
    suite.addTest(unittest.TestLoader().loadTestsFromTestCase(
        VaultSoftDeleteTest))
    return suite



from base import *  # noqa

# Sentry site_id
SITE_ID = 1

RAVEN_CONFIG = {
    'dsn': ''
}

ALLOWED_HOSTS = [
    'www.example.com',
]

VAULTIER.update({
    'raven_key': ''
})

SECRET_KEY = ''

DATABASES = {
    'default': {
        # Add  'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        # Or path to database file if using sqlite3.
        'NAME': 'vaultier',
        # The following settings are not used with sqlite3:
        'USER': 'vaultier',
        'PASSWORD': 'vaultier',
        # Empty for localhost through domain sockets or '127.0.0.1'
        # for localhost through TCP.
        'HOST': '127.0.0.1',
        'PORT': '',  # Set to empty string for default.
    }
}

# url of site used in emailing, templates and so.
SITE_URL = 'http://www.example.com/'

COMPRESS_ENABLED = 1

# Email configuration
EMAIL_HOST = 'localhost'
EMAIL_PORT = 25
EMAIL_HOST_USER = ''
EMAIL_HOST_PASSWORD = ''
EMAIL_USE_TLS = False



import os
import sys
import re
from threading import Lock


def cant_compile():
    blockers = []
    #try:
    #    import spidermonkey
    #except ImportError:
    #    blockers.append('Missing "python-spidermonkey" module')
    return blockers


def cant_observe():
    blockers = []
    if sys.platform != "linux2":
        blockers.append('Available only on Linux platform with '
                        '"pyinotify" module istalled')
    else:
        try:
            import pyinotify  # noqa
        except ImportError:
            blockers.append('Missing "pyinotify" module')
    return blockers


def is_outdated(src_file, compiled_file):
    return not os.path.exists(compiled_file) or \
        int(os.stat(compiled_file).st_mtime * 1000) < \
        int(os.stat(src_file).st_mtime * 1000)


def thread_safe(func):
    lock = Lock()

    def decorated(*args, **kwargs):
        with lock:
            ret = func(*args, **kwargs)
            return ret
    return decorated


class ReadableError(Exception):
    pass


class Console(object):

    colors = {
        "black": "\033[30m", "red":    "\033[31m",
        "green": "\033[32m", "yellow": "\033[33m",
        "blue":  "\033[34m", "purple": "\033[35m",
        "cyan":  "\033[36m", "reset":  "\033[0m"
    }

    def __init__(self, out=None, err=None, raw=False):
        self._out = out or sys.stdout
        self._err = err or sys.stderr
        self.raw = raw
        self.re = re.compile("<color:(%s)>" % "|".join(self.colors.keys()))

    def _colorize(self, match):
        return "" if self.raw else self.colors.get(match.group(1), "")

    def _format(self, s):
        return self.re.sub(self._colorize, "%s<color:reset>\n" % s)

    def set_out(self, out):
        self._out = out

    def set_err(self, err):
        self._err = err

    @thread_safe
    def out(self, s):
        self._out.write(self._format(s))
        self._out.flush()

    @thread_safe
    def err(self, s):
        self._err.write(self._format(s))
        self._err.flush()


class NullConsole(object):
    _stub = lambda *args, **kwargs: None
    __init__ = _stub
    set_out = _stub
    set_err = _stub
    out = _stub
    err = _stub



from django.db import models
from rest_framework.fields import IntegerField


class RecoverTypeField(IntegerField):

    REBUILD = 1
    DISABLE = 2


class MemberStatusField(models.IntegerField):

    STATUS_INVITED = 100
    STATUS_MEMBER_WITHOUT_WORKSPACE_KEY = 200
    STATUS_MEMBER = 300
    STATUS_MEMBER_BROKEN = 400

    STATUS_CHOICES = (
        (STATUS_INVITED, 'INVITED'),
        (STATUS_MEMBER_WITHOUT_WORKSPACE_KEY, 'MEMBER_WITHOUT_WORKSPACE_KEY'),
        (STATUS_MEMBER, 'MEMBER'),
        (STATUS_MEMBER_BROKEN, 'MEMBER_WITH_LOST_KEY')
    )

    def __init__(self, *args, **kwargs):
        kwargs['choices'] = self.STATUS_CHOICES
        kwargs['default'] = 3
        super(MemberStatusField, self).__init__(*args, **kwargs)



from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import ModelViewSet
from libs.version.context import VersionContextAwareApiViewMixin
from vaultier.business.mixins import SoftDeleteModelMixin, \
    RetrieveBySlugMixin, AtomicTransactionMixin
from vaults.business.permissions import CanManageVaultPermission
from vaults.models import Vault
from vaults.serializers import VaultSerializer


class VaultViewSet(AtomicTransactionMixin, RetrieveBySlugMixin,
                   SoftDeleteModelMixin, VersionContextAwareApiViewMixin,
                   ModelViewSet):
    """
    API endpoint that allows vaults to be viewed or edited.
    """
    model = Vault
    permission_classes = (IsAuthenticated, CanManageVaultPermission)
    serializer_class = VaultSerializer
    filter_fields = ('workspace',)

    def pre_save(self, object):
        if object.pk is None:
            self.check_object_permissions(self.request, object)
            object.created_by = self.request.user
        return super(VaultViewSet, self).pre_save(object)

    def get_queryset(self):
        if self.action == 'list':
            queryset = Vault.objects.all_for_user(self.request.user)
        else:
            queryset = Vault.objects.all()
        return queryset



from django.db import models


class RoleLevelField(models.IntegerField):
    LEVEL_CREATE = 50
    LEVEL_READ = 100
    LEVEL_WRITE = 200

    ROLE_CHOICES = (
        (LEVEL_CREATE, 'CREATE'),
        (LEVEL_READ, 'READ'),
        (LEVEL_WRITE, 'WRITE'),
    )

    def __init__(self, *args, **kwargs):
        kwargs['choices'] = self.ROLE_CHOICES
        super(RoleLevelField, self).__init__(*args, **kwargs)


class AclDirectionField(models.IntegerField):
    DIR_UP = -1
    DIR_CURRENT = 0
    DIR_DOWN = 1

    DIR_CHOICES = (
        (DIR_UP, 'UP'),
        (DIR_CURRENT, 'CURRENT'),
        (DIR_DOWN, 'DOWN')
    )

    def __init__(self, *args, **kwargs):
        kwargs['choices'] = self.DIR_CHOICES
        super(AclDirectionField, self).__init__(*args, **kwargs)


class AclLevelField(models.IntegerField):
    LEVEL_READ = 10
    LEVEL_CREATE = 20
    LEVEL_WRITE = 30

    ACL_CHOICES = (
        (LEVEL_CREATE, 'CREATE'),
        (LEVEL_READ, 'READ'),
        (LEVEL_WRITE, 'WRITE'),
    )

    def __init__(self, *args, **kwargs):
        kwargs['choices'] = self.ACL_CHOICES
        super(AclLevelField, self).__init__(*args, **kwargs)



from django.db.transaction import atomic
from rest_framework.exceptions import MethodNotAllowed
from rest_framework.filters import SearchFilter, DjangoFilterBackend, \
    OrderingFilter
from rest_framework.mixins import UpdateModelMixin, RetrieveModelMixin, \
    CreateModelMixin, ListModelMixin
from rest_framework.permissions import IsAuthenticated
from rest_framework.response import Response
from rest_framework.views import APIView
from rest_framework.viewsets import ModelViewSet, GenericViewSet
from accounts.business.authentication import TokenAuthentication, \
    HashAuthentication
from accounts.business.fields import MemberStatusField
from accounts.business.permissions import CanManageUserPermission, \
    LostKeyPermission
from accounts.models import User, LostKey, Member
from accounts.serializers import AuthSerializer, TokenSerializer, \
    UserSerializer, UserKeySerializer, LostKeySerializer, \
    LostKeyCreateSerializer, MemberInviteSerializer, MemberResendSerializer, \
    MemberSerializer
from vaultier.business.exceptions import CustomAPIException
from rest_framework import status
from rest_framework import mixins, viewsets
from vaultier.business.mixins import AtomicTransactionMixin
from workspaces.business.permissions import CanManageMemberPermission
from .business.authentication import Authenticator
from django.conf import settings


class AuthView(APIView):

    @atomic
    def auth(self, request):
        serializer = AuthSerializer(data=request.DATA)
        if serializer.is_valid():
            try:
                token = Authenticator.authenticate(
                    email=serializer.data.get('email'),
                    date=serializer.data.get('date'),
                    signature=serializer.data.get('signature')
                )

                if token:
                    return Response(TokenSerializer(token).data)
                else:
                    return Response({'result': False},
                                    status=status.HTTP_403_FORBIDDEN)
            except Exception as e:
                raise CustomAPIException(exception=e)

        raise CustomAPIException(detail=serializer.errors)

    def post(self, request):
        return self.auth(request)


class UserViewSet(AtomicTransactionMixin,
                  UpdateModelMixin,
                  RetrieveModelMixin,
                  CreateModelMixin,
                  ListModelMixin,
                  GenericViewSet):
    """
    User resource endpoint
    """
    serializer_class = UserSerializer
    permission_classes = (CanManageUserPermission,)
    model = User

    def create(self, request, *args, **kwargs):
        """
        Check if is registration allowed or if is any registered user in here.
        Otherwise raise 405 exception
        """
        if settings.VAULTIER.get('registration_allow') or not \
                bool(User.objects.all().count()):
            return super(UserViewSet, self).create(request, *args, **kwargs)
        raise MethodNotAllowed(method='POST')

    def get_serializer_class(self):
        """
        Change serializer class to UserKeySerializer in list action
        """
        if self.action in ["list"]:
            return UserKeySerializer
        return super(UserViewSet, self).get_serializer_class()


class LostKeyViewSet(AtomicTransactionMixin,
                     mixins.RetrieveModelMixin,
                     mixins.UpdateModelMixin,
                     mixins.CreateModelMixin,
                     viewsets.GenericViewSet):
    """
    LostKey view points
    """
    model = LostKey
    serializer_class = LostKeySerializer
    permission_classes = (LostKeyPermission,)
    authentication_classes = (HashAuthentication,)

    def get_serializer_class(self):
        """
        Dynamically retrieve serializer based on request view
        :return: Serializer
        """
        if self.action == 'create':
            return LostKeyCreateSerializer
        else:
            return LostKeySerializer


class MemberViewSet(AtomicTransactionMixin, ModelViewSet):

    model = Member
    serializer_class = MemberSerializer
    authentication_classes = (TokenAuthentication, )
    permission_classes = (IsAuthenticated, CanManageMemberPermission)
    filter_backends = (SearchFilter, DjangoFilterBackend, OrderingFilter,)
    search_fields = ('invitation_email', 'user__email', 'user__nickname',)
    filter_fields = ('workspace', 'status')
    ordering = ('status', )

    def invite(self, request, *args, **kwargs):
        serializer = MemberInviteSerializer(data=request.DATA)

        if serializer.is_valid():
            member = Member(
                invitation_email=serializer.object.get('email'),
                workspace=serializer.object.get('workspace'),
                created_by=request.user
            )

            self.check_object_permissions(request, member)

            member = Member.objects.invite(
                member,
                send=serializer.object.get('send'),
                resend=serializer.object.get('resend')
            )

            return Response(
                MemberSerializer(member).data,
                status=status.HTTP_200_OK,
            )

        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def resend(self, request, *args, **kwargs):
        serializer = MemberResendSerializer(data=request.DATA)
        if serializer.is_valid():

            member = self.get_object(
                queryset=Member.objects.all_for_user(request.user)
                .filter(status=MemberStatusField.STATUS_INVITED))

            self.check_object_permissions(request, member)

            Member.objects.send_invitation(member)

            return Response(
                MemberSerializer(member).data,
                status=status.HTTP_200_OK,
            )
        return Response(serializer.errors, status=status.HTTP_400_BAD_REQUEST)

    def create(self, request, *args, **kwargs):
        return self.invite(request, *args, **kwargs)

    def update(self, request, *args, **kwargs):
        return self.resend(request, *args, **kwargs)

    def get_queryset(self):
        queryset = Member.objects.all_for_user(self.request.user)
        return queryset



from django.utils.crypto import get_random_string
from logan.runner import run_app
import sys


CONFIG_TEMPLATE = '''"""\nVaultier configuration file\n"""\n
from vaultier.settings.prod import *
import os

RAVEN_CONFIG = {
    'dsn': ''
}

ALLOWED_HOSTS = [
    'localhost',
    '127.0.0.1',
    '$(DOMAIN)',
]

VAULTIER.update({
    'raven_key': '',
    'registration_allow': False,
    'allow_anonymous_usage_statistics': True,
    'from_email': 'noreply@$(DOMAIN)',
})

CONFIG_DIR = os.path.abspath(os.path.dirname(__file__))
MEDIA_ROOT = os.path.join(CONFIG_DIR, 'data')


SECRET_KEY = '$(SECRET_KEY)'

DATABASES = {
    'default': {
        # Add 'postgresql_psycopg2', 'mysql', 'sqlite3' or 'oracle'.
        'ENGINE': 'django.db.backends.$(DB_TYPE)',
        'NAME': 'vaultier', # Or path to database file if using sqlite3.
        # The following settings are not used with sqlite3:
        'USER': 'vaultier',
        'PASSWORD': 'vaultier',
        # Empty for localhost through domain sockets or '127.0.0.1'
        # for localhost through TCP.
        'HOST': '127.0.0.1',
        'PORT': '',  # Set to empty string for default.
    }
}

# url of site used in emailing, templates and so.
SITE_URL = 'https://$(DOMAIN)/'

# Email configuration
EMAIL_HOST = 'localhost'
EMAIL_PORT = 25
EMAIL_HOST_USER = ''
EMAIL_HOST_PASSWORD = ''
EMAIL_USE_TLS = False
'''

_managed = False


def _db_choice():
    """
    Database choice helper method
    """
    t = raw_input(">>> Please enter database type (1 - posgresql, "
                  "2 - mysql, 3 - sqlite, 4 - oracle). [example: 1]: ")
    try:
        t = int(t)
    except ValueError:
        return None
    if t not in [x for x in xrange(1, 5)]:
        return None
    db_driver = {
        1: 'postgresql_psycopg2',
        2: 'mysql',
        3: 'sqlite3',
        4: 'oracle'
    }[t]

    return db_driver


def _generate_settings():
    """
    Generate the settings from template
    """

    # Generate scret key
    chars = 'abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)'
    secret_key = get_random_string(50, chars)

    # Fetch DB type
    db_type = None
    if not _managed:
        while True:
            db_type = _db_choice()
            if db_type:
                break
    else:
        db_type = 'postgresql_psycopg2'

    # Input domain
    domain = None
    if not _managed:
        domain = raw_input(
            ">>> Please enter the domain where your project is going to "
            "reside. [example: vaultier.mydomain.com]: "
        )
    if not domain:
        domain = "www.example.com"

    cfg = CONFIG_TEMPLATE.replace('$(SECRET_KEY)', secret_key). \
        replace('$(DB_TYPE)', str(db_type))

    if _managed:
        # Update application settings
        cfg = cfg.replace(
            "'from_email': 'noreply@$(DOMAIN)'",
            "'from_email': os.getenv('VAULTIER_FROM_EMAIL', "
            "'noreply@{}')".format(domain),
        )
        cfg = cfg.replace(
            "'registration_allow': False",
            "'registration_allow': bool(os.getenv("
            "'VAULTIER_ALLOW_REGISTRATION', False))",
        )
        cfg = cfg.replace(
            "'allow_anonymous_usage_statistics': True",
            "'allow_anonymous_usage_statistics': bool(os.getenv("
            "'VAULTIER_ALLOW_STATISTICS', True))",
        )
        # Update allowed hosts
        cfg = cfg.replace(
            '\'$(DOMAIN)\'',
            "os.getenv('VAULTIER_DOMAIN', 'www.example.com')",
            1
        )
        # Update site url
        cfg = cfg.replace(
            '$(DOMAIN)',
            "' + os.getenv('VAULTIER_DOMAIN', 'www.example.com') + '"
        )

        # Update database settings
        cfg = cfg.replace(
            "'NAME': 'vaultier'",
            "'NAME': os.getenv('VAULTIER_DATABASE_NAME', 'vaultier')")
        cfg = cfg.replace(
            "'USER': 'vaultier'",
            "'USER': os.getenv('VAULTIER_DATABASE_USER', 'vaultier')")
        cfg = cfg.replace(
            "'PASSWORD': 'vaultier'",
            "'PASSWORD': os.getenv('VAULTIER_DATABASE_PASSWORD', 'vaultier')")
        cfg = cfg.replace(
            "'HOST': '127.0.0.1'",
            "'HOST': os.getenv('VAULTIER_DATABASE_HOST', '127.0.0.1')")
        cfg = cfg.replace(
            "'PORT': ''",
            "'PORT': os.getenv('VAULTIER_DATABASE_PORT', '')")


        # Update email settings
        cfg = cfg.replace(
            "EMAIL_HOST = 'localhost'",
            "EMAIL_HOST = os.getenv('VAULTIER_EMAIL_HOST', '')")
        cfg = cfg.replace(
            "EMAIL_PORT = 25",
            "EMAIL_PORT = os.getenv('VAULTIER_EMAIL_PORT', 25)")
        cfg = cfg.replace(
            "EMAIL_HOST_USER = ''",
            "EMAIL_HOST_USER = os.getenv('VAULTIER_EMAIL_USER', '')")
        cfg = cfg.replace(
            "EMAIL_HOST_PASSWORD = ''",
            "EMAIL_HOST_PASSWORD = os.getenv('VAULTIER_EMAIL_PASSWORD', '')")
        cfg = cfg.replace(
            "EMAIL_USE_TLS = False",
            "EMAIL_USE_TLS = bool(os.getenv('VAULTIER_EMAIL_TLS', False))")

        return cfg
    else:
        return cfg.replace('$(DOMAIN)', domain)


def main():
    # Automatically managed install, we need to circumvent logan a bit
    global _managed
    _managed = '--managed' in sys.argv
    if _managed:
        sys.argv.pop(sys.argv.index('--managed'))

    run_app(
        project='vaultier',
        default_config_path='vaultier_conf.py',
        default_settings='vaultier.settings.prod',
        settings_initializer=_generate_settings,
        settings_envvar='VAULTIER_CONF'
    )


# Execute if run from console
if __name__ == '__main__':
    main()



from libs.tree.iterator import AbstractTreeIterator


class CardTreeIterator(AbstractTreeIterator):

    def get_parent_object(self):
        return self.get_object().vault

    def get_child_objects(self):
        return []



from django.conf.urls import patterns, url

from rest_framework import routers
from accounts.api import UserViewSet, LostKeyViewSet, AuthView, MemberViewSet
from acls.api import RoleViewSet
from cards.api import CardViewSet
from news.api import NewsApiView
from search.api import SearchView
from secrets.api import SecretViewSet, SecretBlobViewSet
from vaults.api import VaultViewSet
from workspaces.api import WorkspaceViewSet, WorkspaceKeyViewSet, \
    InvitationViewSet

router = routers.DefaultRouter()
router.register(r'users', UserViewSet, base_name='user')
router.register(r'workspaces', WorkspaceViewSet, base_name='workspace')
router.register(r'vaults', VaultViewSet, base_name='vault')
router.register(r'cards', CardViewSet, base_name='card')
router.register(r'secrets', SecretViewSet, base_name='secret')
router.register(r'secret_blobs', SecretBlobViewSet, base_name='secret_blob')
router.register(r'workspace_keys', WorkspaceKeyViewSet,
                base_name='workspace_key')
router.register(r'members', MemberViewSet, base_name='member')
router.register(r'invitations', InvitationViewSet, base_name='invitation')
router.register(r'roles', RoleViewSet, base_name='role')
router.register(r'lost_keys', LostKeyViewSet, base_name='lost_keys')

urlpatterns = router.urls

urlpatterns += patterns('',
    # news
    url(r'^news/$', NewsApiView.as_view(), name='news-list'),
    # search
    url(r'^search/search$', SearchView.as_view(), name='search-search'),
    # auth
    url(r'^auth/auth$', AuthView.as_view(), name='auth-auth'),
    url(r'^auth/user$', UserViewSet.as_view(), name='auth-user'),
)



from rest_framework.permissions import BasePermission
from acls.business.fields import AclLevelField


def has_object_acl(user, object, level):
    if not object:
        raise RuntimeError('Cannot check acl level for object None')

    # newly created one, always allow. Permissions are determined by
    # related objects in higher code leves
    if not object.id:
        return True

    # existing object, check acls
    for acl in object.acl_set.all():
        if acl.level == level and acl.user == user:
            return True


class CanManageRolePermission(BasePermission):
    def has_object_permission(self, request, view, role):

        object = role.get_object()
        result = has_object_acl(request.user, object,
                                AclLevelField.LEVEL_WRITE)

        return result



from django.core import management
from django.core.management import BaseCommand
from django.db import OperationalError
from django.core.cache import cache
from django.db.utils import ProgrammingError


class Command(BaseCommand):

    def handle(self, *args, **options):
        print ">>> Initializing your database"
        try:
            management.call_command('syncdb')
            management.call_command('migrate')
            try:
                # do we need cache table?
                cache.get('', None)
            except ProgrammingError:
                # yes we do
                management.call_command('createcachetable', 'vaultier_cache')

            # public static files
            management.call_command('collectstatic', interactive=False)
        except OperationalError as e:
            msg = ">>> Your DB is not configured correctly: {}"
            print msg.format(e.message)
        else:
            print ">>> DB is initialized, you can now try to run Vaultier " \
                  "using 'vaultier runserver'"



from rest_framework.permissions import BasePermission
from accounts.models import Member
from acls.business.fields import AclLevelField
from acls.business.permissions import has_object_acl


class CanManageMemberPermission(BasePermission):

    def has_object_permission(self, request, view, obj):

        if view.action == 'destroy':
            workspace = obj.workspace
            return has_object_acl(request.user, workspace,
                                  AclLevelField.LEVEL_WRITE)

        workspace = obj.workspace
        return has_object_acl(request.user, workspace,
                              AclLevelField.LEVEL_READ)


class CanManageWorkspace(BasePermission):

    def has_object_permission(self, request, view, obj):

        if view.action == ['retrieve', 'list']:
            required_level = AclLevelField.LEVEL_READ
        else:
            required_level = AclLevelField.LEVEL_WRITE

        if not obj.pk:
            return True
        else:
            return has_object_acl(request.user, obj, required_level)


class CanManageWorkspaceKey(BasePermission):

    def has_object_permission(self, request, view, obj):
        return Member.objects.\
            all_to_transfer_keys(request.user).filter(id=obj.id).count() > 0



from libs.changes.changes import post_change
from libs.version.context import version_context_manager

ACTION_CREATED = 10
ACTION_UPDATED = 20
ACTION_MOVED = 30
ACTION_SOFTDELETED = 40
ACTION_DELETED = 50

manipulators = {}


def factory_manipulator(version, manipulator_id):
    if version.manipulator_id:
        cls = get_manipulator_class(version.manipulator_id)
        version.manipulator_id = manipulator_id
    else:
        cls = get_manipulator_class(manipulator_id)
        version.manipulator_id = manipulator_id

    return cls(manipulator_id, version)


def get_manipulator_class(id):
    if id in manipulators:
        cls = manipulators[id]
    else:
        msg = 'manipulator class for "{name}" not found'
        raise AttributeError(msg.format(name=id))
    return cls


def register_manipulator_signal(version_cls, manipulator_id=None,
                                condition=None):
    if not condition:
        raise AttributeError('condition kwarg is required')

    def callback(signal=None, sender=None, instance=None, event_type=None,
                 overwritten_values=None, **kwargs):

        overwritten_values = condition.will_do_version(
            signal=signal,
            sender=sender,
            instance=instance,
            event_type=event_type,
            overwritten_values=overwritten_values,
            **kwargs
        )

        if overwritten_values:
            version = version_cls(versioned=instance)
            manipulator = factory_manipulator(version, manipulator_id)
            manipulator.store_state(overwritten_values, instance)
            manipulator.save()

    post_change.connect(callback, sender=condition.required_sender, weak=False)


def register_manipulator_class(name, cls):
    manipulators[name] = cls


class VersionManipulator(object):
    version = None
    action_name = 'action'
    action_id = 0

    def __init__(self, id, version, **kwargs):
        self.id = id
        self.version = version
        self.version.manipulator_cls = self.id
        super(VersionManipulator, self).__init__(**kwargs)

    def determine_action_name(self):
        return self.action_name

    def determine_versioned_related(self):
        return self.version.versioned

    def determine_action_id(self):
        return self.action_id

    def store_state(self, revert_data, model):
        self.version.revert_data = revert_data
        self.version.revert_fields = revert_data.keys()

        self.version.action_name = self.determine_action_name()
        self.version.action_id = self.determine_action_id()
        self.version.versioned_related = self.determine_versioned_related()
        self.version.created_by = version_context_manager.get_user()

    def get_diff(self, fields=None):
        result = {}
        version = self.version
        versioned = self.version.versioned
        if self.can_revert():
            if fields:
                fields = set(version.revert_fields.keys()).intersection(fields)
            else:
                fields = version.revert_fields.keys()

            for field in fields:
                result[field] = {
                    'from': getattr(versioned, field),
                    'to': version.revert_data[field],
                }
        return result

    def save(self):
        return self.version.save()

    def revert(self):
        pass

    def can_revert(self):
        return True


class ModelCreatedManipulator(VersionManipulator):
    action_name = 'created'
    action_id = ACTION_CREATED

    def store_state(self, revert_data, model):
        super(ModelCreatedManipulator, self).store_state(revert_data, model)
        self.version.revert_fields = {}
        self.version.revert_data = {}

    def can_revert(self):
        return False


class ModelUpdatedManipulator(VersionManipulator):
    action_name = 'updated'
    action_id = ACTION_UPDATED


class ModelMovedManipulator(VersionManipulator):
    action_name = 'moved'
    action_id = ACTION_MOVED


class ModelSoftDeletedManipulator(VersionManipulator):
    action_name = 'softdeleted'
    action_id = ACTION_SOFTDELETED

    def determine_versioned_related(self):
        return self.version.versioned.get_tree_iterator().get_parent_object()
